﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Threading.Tasks;
using AngleSharp.Parser.Html;
using Jackett.Models;
using Jackett.Models.IndexerConfig;
using Jackett.Services.Interfaces;
using Jackett.Utils;
using Jackett.Utils.Clients;
using Newtonsoft.Json.Linq;
using NLog;

namespace Jackett.Indexers
{
    public class SevenTor : BaseWebIndexer
    {
        private string LoginUrl { get { return SiteLink + "ucp.php?mode=login"; } }
        private string SearchUrl { get { return SiteLink + "search.php"; } }

        private new ConfigurationDataBasicLoginWithRSSAndDisplay configData
        {
            get { return (ConfigurationDataBasicLoginWithRSSAndDisplay)base.configData; }
            set { base.configData = value; }
        }

        public SevenTor(IIndexerConfigurationService configService, WebClient wc, Logger l, IProtectionService ps)
            : base(name: "7tor",
                   description: "7Tor is a RUSSIAN Semi-Private site for MOVIES / TV / GENERAL",
                   link: "https://7tor.org/",
                   caps: TorznabUtil.CreateDefaultTorznabTVCaps(),
                   configService: configService,
                   client: wc,
                   logger: l,
                   p: ps,
                   configData: new ConfigurationDataBasicLoginWithRSSAndDisplay())
        {
            Encoding = Encoding.UTF8;
            Language = "ru-ru";
            Type = "semi-private";

            AddCategoryMapping(30, TorznabCatType.Movies, "Video content");
            AddCategoryMapping(31, TorznabCatType.TVDocumentary, "   Documentary films, TV and other video");
            AddCategoryMapping(127, TorznabCatType.TVDocumentary, "      Documentary movies and TV shows");
            AddCategoryMapping(1071, TorznabCatType.TVDocumentary, "         Documentary (DVD)");
            AddCategoryMapping(1069, TorznabCatType.TVDocumentary, "         Documentary (HD Video)");
            AddCategoryMapping(1070, TorznabCatType.TVDocumentary, "            TV shows (HD Video), non-documentary");
            AddCategoryMapping(1843, TorznabCatType.TVDocumentary, "            Biographies. Personality and idols (HD Video)");
            AddCategoryMapping(1844, TorznabCatType.TVDocumentary, "            Military Science (HD Video)");
            AddCategoryMapping(1845, TorznabCatType.TVDocumentary, "            Natural science, science and technology (HD Video)");
            AddCategoryMapping(1846, TorznabCatType.TVDocumentary, "            Travel and Tourism (HD Video)");
            AddCategoryMapping(1847, TorznabCatType.TVDocumentary, "            Flora and fauna (HD Video)");
            AddCategoryMapping(1848, TorznabCatType.TVDocumentary, "            History (HD Video)");
            AddCategoryMapping(1849, TorznabCatType.TVDocumentary, "            BBC, Discovery, National Geographic (HD Video)");
            AddCategoryMapping(1850, TorznabCatType.TVDocumentary, "            Crime Documentary (HD Video)");
            AddCategoryMapping(1072, TorznabCatType.TVDocumentary, "         Biographies. Personality and idols");
            AddCategoryMapping(1073, TorznabCatType.TVDocumentary, "         Documentary movies and TV shows on film and animation (including biographies)");
            AddCategoryMapping(1074, TorznabCatType.TVDocumentary, "         Art, Art History");
            AddCategoryMapping(1075, TorznabCatType.TVDocumentary, "         Documentaries and television music (including biographies)");
            AddCategoryMapping(1076, TorznabCatType.TVDocumentary, "         criminal documentary");
            AddCategoryMapping(1077, TorznabCatType.TVDocumentary, "         Secrets of the Ages / Special Services / Conspiracy Theory");
            AddCategoryMapping(1078, TorznabCatType.TVDocumentary, "         Movies and TV shows on military issues");
            AddCategoryMapping(1079, TorznabCatType.TVDocumentary, "            The Second World War");
            AddCategoryMapping(1675, TorznabCatType.TVDocumentary, "            Fleet");
            AddCategoryMapping(1080, TorznabCatType.TVDocumentary, "         Accidents / Accidents / Disasters");
            AddCategoryMapping(1081, TorznabCatType.TVDocumentary, "         Aviation (video)");
            AddCategoryMapping(1674, TorznabCatType.TVDocumentary, "            Wings of Russia");
            AddCategoryMapping(1082, TorznabCatType.TVDocumentary, "         Space (Video)");
            AddCategoryMapping(576, TorznabCatType.TVDocumentary, "         Popular-science film");
            AddCategoryMapping(1083, TorznabCatType.TVDocumentary, "         The flora and fauna of the (video)");
            AddCategoryMapping(1084, TorznabCatType.TVDocumentary, "         Travel and Tourism (video)");
            AddCategoryMapping(1085, TorznabCatType.TVDocumentary, "         Social talk show");
            AddCategoryMapping(1086, TorznabCatType.TVDocumentary, "         Information-analytical and socio-political programs");
            AddCategoryMapping(1087, TorznabCatType.TVDocumentary, "         Architecture and Construction (video)");
            AddCategoryMapping(1088, TorznabCatType.TVDocumentary, "         All about home, life and design");
            AddCategoryMapping(1094, TorznabCatType.TVDocumentary, "         The era of the Soviet Union (video)");
            AddCategoryMapping(1095, TorznabCatType.TVDocumentary, "         Battle of psychics / Theory improbability / Seekers / Galileo");
            AddCategoryMapping(1096, TorznabCatType.TVDocumentary, "         Russian sensation / Program Maximum / Profession Reporter / Ukrainian sensation");
            AddCategoryMapping(1097, TorznabCatType.TVDocumentary, "         Paranormal activity");
            AddCategoryMapping(1098, TorznabCatType.TVDocumentary, "         Alternative history, Science (video)");
            AddCategoryMapping(1099, TorznabCatType.TVDocumentary, "         Vnezhanrovaya documentary");
            AddCategoryMapping(1660, TorznabCatType.TVDocumentary, "         Foreign TV-brands");
            AddCategoryMapping(1089, TorznabCatType.TVDocumentary, "            BBC");
            AddCategoryMapping(1090, TorznabCatType.TVDocumentary, "            Discovery");
            AddCategoryMapping(1091, TorznabCatType.TVDocumentary, "            National Geographic");
            AddCategoryMapping(1661, TorznabCatType.TVDocumentary, "            Animal Planet");
            AddCategoryMapping(1662, TorznabCatType.TVDocumentary, "            Da Vinci Learning");
            AddCategoryMapping(1663, TorznabCatType.TVDocumentary, "            History Channel");
            AddCategoryMapping(1664, TorznabCatType.TVDocumentary, "            PBS");
            AddCategoryMapping(1665, TorznabCatType.TVDocumentary, "            Readers Digest");
            AddCategoryMapping(1666, TorznabCatType.TVDocumentary, "            I wonder about everything");
            AddCategoryMapping(1667, TorznabCatType.TVDocumentary, "            Mega-Projects");
            AddCategoryMapping(1668, TorznabCatType.TVDocumentary, "            Prehistoric world");
            AddCategoryMapping(1669, TorznabCatType.TVDocumentary, "            World of Tomorrow");
            AddCategoryMapping(1670, TorznabCatType.TVDocumentary, "            Jacques Cousteau Odyssey");
            AddCategoryMapping(1671, TorznabCatType.TVDocumentary, "            Secrets and Mysteries");
            AddCategoryMapping(1672, TorznabCatType.TVDocumentary, "         History");
            AddCategoryMapping(1092, TorznabCatType.TVDocumentary, "            History: Ancient World / Antiquity / Middle Ages (video)");
            AddCategoryMapping(1093, TorznabCatType.TVDocumentary, "            History: modern and contemporary times");
            AddCategoryMapping(1673, TorznabCatType.TVDocumentary, "         Relax, landscape film");
            AddCategoryMapping(1100, TorznabCatType.TVDocumentary, "         Miscellaneous / nekonditsiya (documentary and transfer)");
            AddCategoryMapping(569, TorznabCatType.TV, "      Entertaining TV programs and shows, fun and humor");
            AddCategoryMapping(1101, TorznabCatType.TV, "         Mind games and quizzes");
            AddCategoryMapping(1102, TorznabCatType.TV, "         Reality and talk show host / category / impressions");
            AddCategoryMapping(1103, TorznabCatType.TV, "         children's TV Shows");
            AddCategoryMapping(1104, TorznabCatType.TV, "         KVN");
            AddCategoryMapping(1105, TorznabCatType.TV, "         Drink Post");
            AddCategoryMapping(1106, TorznabCatType.TV, "         Distorting Mirror / town / in the town");
            AddCategoryMapping(1107, TorznabCatType.TV, "         ice show");
            AddCategoryMapping(1108, TorznabCatType.TV, "         Thank God you came!");
            AddCategoryMapping(1109, TorznabCatType.TV, "         dinner Party");
            AddCategoryMapping(1110, TorznabCatType.TV, "         Good jokes");
            AddCategoryMapping(1111, TorznabCatType.TV, "         Evening Quarter");
            AddCategoryMapping(1112, TorznabCatType.TV, "         Films with a funny transfer (parody)");
            AddCategoryMapping(1113, TorznabCatType.TV, "         Stand-up comedy");
            AddCategoryMapping(1114, TorznabCatType.TV, "         Moment of glory");
            AddCategoryMapping(1115, TorznabCatType.TV, "         Ukrainian Show");
            AddCategoryMapping(1116, TorznabCatType.TV, "         Star Factory");
            AddCategoryMapping(1117, TorznabCatType.TV, "         Dance shows, concerts, performances");
            AddCategoryMapping(1118, TorznabCatType.TV, "         Circus");
            AddCategoryMapping(1119, TorznabCatType.TV, "         School for Scandal");
            AddCategoryMapping(1120, TorznabCatType.TV, "         Satirists and humorists");
            AddCategoryMapping(1873, TorznabCatType.TV, "         Musical show");
            AddCategoryMapping(1121, TorznabCatType.TV, "         Humorous audio transmission");
            AddCategoryMapping(1122, TorznabCatType.TV, "         Audio and video clips (Jokes and humor)");
            AddCategoryMapping(32, TorznabCatType.Movies, "   Foreign movies");
            AddCategoryMapping(567, TorznabCatType.Movies, "      Foreign films 2016");
            AddCategoryMapping(37, TorznabCatType.Movies, "      Foreign films 2011 - 2015 the year");
            AddCategoryMapping(38, TorznabCatType.Movies, "      Foreign films of the year 2006-2010");
            AddCategoryMapping(39, TorznabCatType.Movies, "      Foreign films of the year 2001-2005");
            AddCategoryMapping(40, TorznabCatType.Movies, "      Foreign films 1991-2000");
            AddCategoryMapping(1031, TorznabCatType.Movies, "      Foreign films until 1990");
            AddCategoryMapping(41, TorznabCatType.Movies, "      Classic foreign film");
            AddCategoryMapping(1044, TorznabCatType.Movies, "         Classic foreign film (DVD Video)");
            AddCategoryMapping(1042, TorznabCatType.Movies, "         Classic foreign film (HD Video)");
            AddCategoryMapping(1051, TorznabCatType.Movies, "      Foreign films (DVD)");
            AddCategoryMapping(43, TorznabCatType.Movies, "      Foreign films (HD Video)");
            AddCategoryMapping(773, TorznabCatType.Movies, "      Grindhouse");
            AddCategoryMapping(1040, TorznabCatType.Movies, "         Grindhouse DVD and HD Video");
            AddCategoryMapping(913, TorznabCatType.Movies, "      Asian movies");
            AddCategoryMapping(1010, TorznabCatType.MoviesSD, "         Asian movies (DVD Video)");
            AddCategoryMapping(1052, TorznabCatType.MoviesHD, "         Asian films (HD Video)");
            AddCategoryMapping(1032, TorznabCatType.Movies, "      Indian film");
            AddCategoryMapping(1043, TorznabCatType.Movies, "         Indian Cinema DVD and HD Video");
            AddCategoryMapping(1039, TorznabCatType.Movies, "      Shorts");
            AddCategoryMapping(1041, TorznabCatType.Movies, "      Sound track and Translations");
            AddCategoryMapping(1804, TorznabCatType.Movies, "      Foreign films without translation");
            AddCategoryMapping(1805, TorznabCatType.Movies, "         Foreign films in the original");
            AddCategoryMapping(1806, TorznabCatType.Movies, "            Foreign films in the original (HD)");
            AddCategoryMapping(1807, TorznabCatType.Movies, "         Foreign films with translation into other languages");
            AddCategoryMapping(33, TorznabCatType.Movies, "   national cinema");
            AddCategoryMapping(568, TorznabCatType.Movies, "      Domestic films 2016");
            AddCategoryMapping(44, TorznabCatType.Movies, "      Domestic films of 2011 - 2015 the year");
            AddCategoryMapping(45, TorznabCatType.Movies, "      Domestic films of the year 2006-2010");
            AddCategoryMapping(46, TorznabCatType.Movies, "      Domestic films of the year 2001-2005");
            AddCategoryMapping(47, TorznabCatType.Movies, "      Domestic films of the year 1992-2000");
            AddCategoryMapping(48, TorznabCatType.Movies, "      Cinema of the USSR, Soviet Russia, the Russian republic (1917-1991)");
            AddCategoryMapping(1609, TorznabCatType.Movies, "      Films of the Russian Empire (until 1917)");
            AddCategoryMapping(1048, TorznabCatType.Movies, "      National cinema (DVD)");
            AddCategoryMapping(49, TorznabCatType.Movies, "      National cinema (HD Video)");
            AddCategoryMapping(1046, TorznabCatType.Movies, "      Author debuts");
            AddCategoryMapping(1047, TorznabCatType.Movies, "      Child domestic films");
            AddCategoryMapping(1011, TorznabCatType.Movies, "   Art-house cinema and author");
            AddCategoryMapping(1012, TorznabCatType.Movies, "      Art-house and auteur cinema (DVD)");
            AddCategoryMapping(1038, TorznabCatType.Movies, "      Art-house and auteur cinema (HD Video)");
            AddCategoryMapping(1033, TorznabCatType.Movies, "      Author cinema");
            AddCategoryMapping(1035, TorznabCatType.Movies, "      Shorts (Art-house cinema and author)");
            AddCategoryMapping(1036, TorznabCatType.Movies, "      Documentaries (Art-house cinema and author)");
            AddCategoryMapping(1037, TorznabCatType.Movies, "      Animation (Art-house cinema and author)");
            AddCategoryMapping(1617, TorznabCatType.Movies, "      Intelligent movie");
            AddCategoryMapping(34, TorznabCatType.TV, "   TV series");
            AddCategoryMapping(51, TorznabCatType.TV, "      Domestic series");
            AddCategoryMapping(1860, TorznabCatType.TV, "         Domestic series 2016");
            AddCategoryMapping(1810, TorznabCatType.TV, "         Domestic series 2015");
            AddCategoryMapping(574, TorznabCatType.TV, "         Domestic series 2014");
            AddCategoryMapping(50, TorznabCatType.TV, "      Foreign TV series");
            AddCategoryMapping(1861, TorznabCatType.TV, "         Foreign series 2016");
            AddCategoryMapping(1809, TorznabCatType.TV, "         Foreign series 2015");
            AddCategoryMapping(575, TorznabCatType.TV, "         Foreign series 2014");
            AddCategoryMapping(1181, TorznabCatType.TV, "         Foreign TV shows (HD Video)");
            AddCategoryMapping(1184, TorznabCatType.TV, "         Soaps Spain, Italy, Latin America, Turkey and India");
            AddCategoryMapping(1185, TorznabCatType.TV, "            Indian series");
            AddCategoryMapping(1186, TorznabCatType.TV, "            spanish series");
            AddCategoryMapping(1187, TorznabCatType.TV, "            Italian TV series");
            AddCategoryMapping(1615, TorznabCatType.TV, "            Latin American soap operas");
            AddCategoryMapping(1189, TorznabCatType.TV, "               Official short version Latin American serials");
            AddCategoryMapping(1190, TorznabCatType.TV, "               Latin American soap operas with the voice acting (folders distribution)");
            AddCategoryMapping(1191, TorznabCatType.TV, "               Latin American serials with subtitles");
            AddCategoryMapping(1188, TorznabCatType.TV, "            turkish TV series");
            AddCategoryMapping(1192, TorznabCatType.TV, "            Serials OST Spain, Italy, Latin America, Turkey and India");
            AddCategoryMapping(1193, TorznabCatType.TV, "            For sub-standard hands");
            AddCategoryMapping(1194, TorznabCatType.TV, "         Asian series");
            AddCategoryMapping(1195, TorznabCatType.TV, "            Chinese serials with subtitles");
            AddCategoryMapping(1196, TorznabCatType.TV, "            Korean soap operas with the voice acting");
            AddCategoryMapping(1197, TorznabCatType.TV, "            Korean serials with subtitles");
            AddCategoryMapping(1198, TorznabCatType.TV, "            Other Asian series with the voice acting");
            AddCategoryMapping(1199, TorznabCatType.TV, "            Taiwanese serials with subtitles");
            AddCategoryMapping(1200, TorznabCatType.TV, "            Japanese serials with subtitles");
            AddCategoryMapping(1201, TorznabCatType.TV, "            Japanese TV series with the voice acting");
            AddCategoryMapping(1202, TorznabCatType.TV, "            VMV and others. Videos");
            AddCategoryMapping(1203, TorznabCatType.TV, "            OST Asian series");
            AddCategoryMapping(1616, TorznabCatType.TV, "         Soaps with Ukrainian sound track");
            AddCategoryMapping(1049, TorznabCatType.TV, "   Theater");
            AddCategoryMapping(1050, TorznabCatType.TV, "      Benefit. Master of Arts of Russian theater and cinema.");
            AddCategoryMapping(1053, TorznabCatType.Movies3D, "   3D / Stereo (Cinema, Animation, Video, TV &amp; Sports)");
            AddCategoryMapping(1054, TorznabCatType.Movies3D, "      3D Movies");
            AddCategoryMapping(581, TorznabCatType.Movies3D, "         Foreign Movies 3D");
            AddCategoryMapping(1614, TorznabCatType.Movies3D, "            Asian Movies 3D");
            AddCategoryMapping(1613, TorznabCatType.Movies3D, "         Domestic 3D Movies");
            AddCategoryMapping(586, TorznabCatType.Movies3D, "      3D Animation");
            AddCategoryMapping(1055, TorznabCatType.Movies3D, "      3D Documentaries");
            AddCategoryMapping(1056, TorznabCatType.Movies3D, "      3D Sports");
            AddCategoryMapping(1057, TorznabCatType.Movies3D, "      3D Clips, Music Videos, Movie Trailers");
            AddCategoryMapping(53, TorznabCatType.TVAnime, "   Animation and cartoons");
            AddCategoryMapping(341, TorznabCatType.TVAnime, "      Foreign cartoons");
            AddCategoryMapping(344, TorznabCatType.TVAnime, "         Foreign cartoons (DVD)");
            AddCategoryMapping(342, TorznabCatType.TVAnime, "      Domestic cartoons");
            AddCategoryMapping(1062, TorznabCatType.TVAnime, "         Domestic full-length cartoons");
            AddCategoryMapping(1061, TorznabCatType.TVAnime, "         Domestic cartoons (DVD)");
            AddCategoryMapping(346, TorznabCatType.TVAnime, "      short cartoons");
            AddCategoryMapping(1058, TorznabCatType.TVAnime, "         Short Film (HD Video)");
            AddCategoryMapping(1060, TorznabCatType.TVAnime, "         Foreign short cartoons");
            AddCategoryMapping(1059, TorznabCatType.TVAnime, "         Foreign Short Film (DVD)");
            AddCategoryMapping(345, TorznabCatType.TVAnime, "      Cartoons HD-Video");
            AddCategoryMapping(1063, TorznabCatType.TVAnime, "      cartoon Puzzle");
            AddCategoryMapping(343, TorznabCatType.TVAnime, "      Serial cartoons");
            AddCategoryMapping(1813, TorznabCatType.TVAnime, "      Cartoons and cartoons without translation");
            AddCategoryMapping(1814, TorznabCatType.TVAnime, "         Cartoons and cartoons with the Ukrainian sound track");
            AddCategoryMapping(1064, TorznabCatType.TVAnime, "      Archive and nekonditsiya cartoons and animated series");
            AddCategoryMapping(54, TorznabCatType.TVAnime, "   Anime and everything associated with him");
            AddCategoryMapping(55, TorznabCatType.TVAnime, "      Anime (Main)");
            AddCategoryMapping(976, TorznabCatType.TVAnime, "      Anime (pleerny subsection)");
            AddCategoryMapping(977, TorznabCatType.TVAnime, "      Anime (QC subsection)");
            AddCategoryMapping(333, TorznabCatType.TVAnime, "      Anime DVD-Video");
            AddCategoryMapping(334, TorznabCatType.TVAnime, "      Anime (HD and Blu-ray)");
            AddCategoryMapping(1815, TorznabCatType.TVAnime, "      OST to Anime");
            AddCategoryMapping(979, TorznabCatType.TVAnime, "         Anime OST to (lossless)");
            AddCategoryMapping(335, TorznabCatType.TVAnime, "         OST to Anime (mp3 and others lossy-format)");
            AddCategoryMapping(336, TorznabCatType.TVAnime, "      Manga and other art");
            AddCategoryMapping(474, TorznabCatType.TVAnime, "         Manga");
            AddCategoryMapping(680, TorznabCatType.TVAnime, "         Wallpapers, artbook, and others.");
            AddCategoryMapping(60, TorznabCatType.TVAnime, "      Anime (Hentai)");
            AddCategoryMapping(978, TorznabCatType.TVAnime, "      AMV etc. Videos");
            AddCategoryMapping(980, TorznabCatType.TVAnime, "      Japanese cartoons");
            AddCategoryMapping(1065, TorznabCatType.TVAnime, "      Archive and nekonditsiya Anime");
            AddCategoryMapping(922, TorznabCatType.Movies, "   Faith and Religion (Video)");
            AddCategoryMapping(1068, TorznabCatType.Movies, "      Islam (video)");
            AddCategoryMapping(1067, TorznabCatType.Movies, "      Cults and new religious movements (video)");
            AddCategoryMapping(1066, TorznabCatType.Movies, "      Religions of India, Tibet and East Asia (video)");
            AddCategoryMapping(923, TorznabCatType.Movies, "      Christianity (video)");
            AddCategoryMapping(577, TorznabCatType.TVSport, "   Health &amp; Sports (sports tournaments, films and programs etc)");
            AddCategoryMapping(583, TorznabCatType.TVSport, "      wrestling");
            AddCategoryMapping(740, TorznabCatType.TVSport, "         Professional Wrestling");
            AddCategoryMapping(1176, TorznabCatType.TVSport, "         Independent Wrestling");
            AddCategoryMapping(1177, TorznabCatType.TVSport, "         International Wrestling");
            AddCategoryMapping(1178, TorznabCatType.TVSport, "         Oldschool Wrestling");
            AddCategoryMapping(1179, TorznabCatType.TVSport, "         Documentary Wrestling");
            AddCategoryMapping(677, TorznabCatType.TVSport, "      cycle racing");
            AddCategoryMapping(724, TorznabCatType.TVSport, "      Tennis");
            AddCategoryMapping(925, TorznabCatType.TVSport, "      Athletics / Water Sports");
            AddCategoryMapping(926, TorznabCatType.TVSport, "      motorcycling");
            AddCategoryMapping(930, TorznabCatType.TVSport, "      Hockey");
            AddCategoryMapping(1171, TorznabCatType.TVSport, "         Hockey / Bandy");
            AddCategoryMapping(1172, TorznabCatType.TVSport, "         International hockey tournaments");
            AddCategoryMapping(1173, TorznabCatType.TVSport, "         KXL");
            AddCategoryMapping(932, TorznabCatType.TVSport, "         NHL (until 2011/12)");
            AddCategoryMapping(931, TorznabCatType.TVSport, "         NHL (2013)");
            AddCategoryMapping(1174, TorznabCatType.TVSport, "         USSR - Canada");
            AddCategoryMapping(1175, TorznabCatType.TVSport, "         Documentaries and Analysis (hockey)");
            AddCategoryMapping(1123, TorznabCatType.TVSport, "      Motorsports");
            AddCategoryMapping(1125, TorznabCatType.TVSport, "      Formula 1");
            AddCategoryMapping(1126, TorznabCatType.TVSport, "         Formula 1 2012-2015");
            AddCategoryMapping(1127, TorznabCatType.TVSport, "         Formula January 2016");
            AddCategoryMapping(1128, TorznabCatType.TVSport, "      Volleyball / Handball");
            AddCategoryMapping(1129, TorznabCatType.TVSport, "      Billiards");
            AddCategoryMapping(1130, TorznabCatType.TVSport, "      Poker");
            AddCategoryMapping(1131, TorznabCatType.TVSport, "      Bodybuilding / Power Sports");
            AddCategoryMapping(1132, TorznabCatType.TVSport, "      Boxing");
            AddCategoryMapping(1133, TorznabCatType.TVSport, "      Classic arts");
            AddCategoryMapping(1134, TorznabCatType.TVSport, "      MMA and K-1");
            AddCategoryMapping(1135, TorznabCatType.TVSport, "      American football");
            AddCategoryMapping(1136, TorznabCatType.TVSport, "      Rugby");
            AddCategoryMapping(1137, TorznabCatType.TVSport, "      Baseball");
            AddCategoryMapping(1138, TorznabCatType.TVSport, "      Badminton / Table Tennis");
            AddCategoryMapping(1139, TorznabCatType.TVSport, "      Gymnastics / Dance Competitions");
            AddCategoryMapping(1140, TorznabCatType.TVSport, "      Winter sports");
            AddCategoryMapping(1141, TorznabCatType.TVSport, "      Figure skating");
            AddCategoryMapping(1142, TorznabCatType.TVSport, "      Biathlon");
            AddCategoryMapping(1143, TorznabCatType.TVSport, "      Extreme sports");
            AddCategoryMapping(1144, TorznabCatType.TVSport, "      Football");
            AddCategoryMapping(1146, TorznabCatType.TVSport, "         Russia 2015-2016");
            AddCategoryMapping(1145, TorznabCatType.TVSport, "         Russia 2014-2015");
            AddCategoryMapping(1147, TorznabCatType.TVSport, "         Russia / USSR");
            AddCategoryMapping(1148, TorznabCatType.TVSport, "         England");
            AddCategoryMapping(1149, TorznabCatType.TVSport, "         Spain");
            AddCategoryMapping(1150, TorznabCatType.TVSport, "         Italy");
            AddCategoryMapping(1151, TorznabCatType.TVSport, "         Germany");
            AddCategoryMapping(1851, TorznabCatType.TVSport, "         France");
            AddCategoryMapping(1152, TorznabCatType.TVSport, "         Ukraine");
            AddCategoryMapping(1153, TorznabCatType.TVSport, "         Other national championships and cups");
            AddCategoryMapping(1154, TorznabCatType.TVSport, "         International football tournaments");
            AddCategoryMapping(1157, TorznabCatType.TVSport, "         European Cups");
            AddCategoryMapping(1156, TorznabCatType.TVSport, "            Eurocup 2011-2014");
            AddCategoryMapping(1155, TorznabCatType.TVSport, "            Eurocup 2014-2015");
            AddCategoryMapping(1161, TorznabCatType.TVSport, "            Eurocup 2015-2016");
            AddCategoryMapping(1158, TorznabCatType.TVSport, "         European Championships");
            AddCategoryMapping(1159, TorznabCatType.TVSport, "            European Championship 2016");
            AddCategoryMapping(1863, TorznabCatType.TVSport, "               European Championships 2016 (Selection section)");
            AddCategoryMapping(1864, TorznabCatType.TVSport, "               European Championship 2016 (the final part)");
            AddCategoryMapping(1160, TorznabCatType.TVSport, "         World Championships");
            AddCategoryMapping(1852, TorznabCatType.TVSport, "            World Championship 2018");
            AddCategoryMapping(1162, TorznabCatType.TVSport, "         Friendly tournaments and matches");
            AddCategoryMapping(1163, TorznabCatType.TVSport, "         The survey and analysis of transmission");
            AddCategoryMapping(1853, TorznabCatType.TVSport, "            The survey and analytical programs 2014-2016");
            AddCategoryMapping(1164, TorznabCatType.TVSport, "         Mini Soccer / Football");
            AddCategoryMapping(1165, TorznabCatType.TVSport, "      Basketball");
            AddCategoryMapping(1166, TorznabCatType.TVSport, "         International competitions");
            AddCategoryMapping(1167, TorznabCatType.TVSport, "         NBA / NCAA (until 2000)");
            AddCategoryMapping(1168, TorznabCatType.TVSport, "         NBA / NCAA (2000-2010 biennium).");
            AddCategoryMapping(1169, TorznabCatType.TVSport, "         NBA / NCAA (2010-2016 biennium).");
            AddCategoryMapping(1170, TorznabCatType.TVSport, "         European club basketball");
            AddCategoryMapping(1885, TorznabCatType.TVSport, "      XXXI Summer Olympic Games. Rio de Janeiro 2016");
            AddCategoryMapping(1886, TorznabCatType.TVSport, "         Football");
            AddCategoryMapping(1887, TorznabCatType.TVSport, "         Basketball");
            AddCategoryMapping(1888, TorznabCatType.TVSport, "         Volleyball / Beach Volleyball / Handball / Water Polo");
            AddCategoryMapping(1889, TorznabCatType.TVSport, "         Athletics");
            AddCategoryMapping(1890, TorznabCatType.TVSport, "         Tennis / Table Tennis / Badminton");
            AddCategoryMapping(1891, TorznabCatType.TVSport, "         Boxing / Martial Arts and Martial Arts / Weightlifting");
            AddCategoryMapping(1892, TorznabCatType.TVSport, "         Water Sports / Boating");
            AddCategoryMapping(1893, TorznabCatType.TVSport, "         cycle racing");
            AddCategoryMapping(1894, TorznabCatType.TVSport, "         Gymnastics");
            AddCategoryMapping(1895, TorznabCatType.TVSport, "         Other Sports");
            AddCategoryMapping(1896, TorznabCatType.TVSport, "         The survey and analysis of transmission");
            AddCategoryMapping(1897, TorznabCatType.Books, "         Books, manuals, periodicals on the Olympic theme");
            AddCategoryMapping(1575, TorznabCatType.Movies, "   Video for mobile devices");
            AddCategoryMapping(1576, TorznabCatType.Movies, "      Video for Smartphones and PDAs");
            AddCategoryMapping(1577, TorznabCatType.Movies, "      Mobile Video (3GP)");
            AddCategoryMapping(1589, TorznabCatType.Movies, "   Video for Apple devices");
            AddCategoryMapping(1590, TorznabCatType.Movies, "      Video (Apple)");
            AddCategoryMapping(1592, TorznabCatType.MoviesSD, "         Movies for iPod, iPhone, iPad");
            AddCategoryMapping(1593, TorznabCatType.MoviesSD, "         Soaps for iPod, iPhone, iPad");
            AddCategoryMapping(1594, TorznabCatType.MoviesSD, "         Cartoons to iPod, iPhone, iPad");
            AddCategoryMapping(1595, TorznabCatType.MoviesSD, "         Anime for iPod, iPhone, iPad");
            AddCategoryMapping(1596, TorznabCatType.MoviesSD, "         The music video for iPod, iPhone, iPad");
            AddCategoryMapping(1591, TorznabCatType.MoviesHD, "      Videos HD (Apple)");
            AddCategoryMapping(1597, TorznabCatType.MoviesHD, "         HD Movies to Apple TV");
            AddCategoryMapping(1598, TorznabCatType.MoviesHD, "         HD TV Shows on Apple TV");
            AddCategoryMapping(1599, TorznabCatType.MoviesHD, "         Cartoon HD for Apple TV");
            AddCategoryMapping(1600, TorznabCatType.MoviesHD, "         Documentary HD video for Apple TV");
            AddCategoryMapping(1601, TorznabCatType.MoviesHD, "         Music HD video for Apple TV");
            AddCategoryMapping(1568, TorznabCatType.Movies, "   Trailers and additional materials for films");
            AddCategoryMapping(1549, TorznabCatType.Movies, "   Video consoles");
            AddCategoryMapping(1550, TorznabCatType.Movies, "      Video for PSVita");
            AddCategoryMapping(1551, TorznabCatType.Movies, "      Movies for PSP");
            AddCategoryMapping(1552, TorznabCatType.Movies, "      for PSP TV Shows");
            AddCategoryMapping(1553, TorznabCatType.Movies, "      Cartoons for PSP");
            AddCategoryMapping(1554, TorznabCatType.Movies, "      Drama for PSP");
            AddCategoryMapping(1555, TorznabCatType.Movies, "      Anime for PSP");
            AddCategoryMapping(1556, TorznabCatType.Movies, "      Video to PSP");
            AddCategoryMapping(1557, TorznabCatType.Movies, "      Videos for the PS3 and other consoles");
            AddCategoryMapping(165, TorznabCatType.Movies, "   video Game");
            AddCategoryMapping(1544, TorznabCatType.Movies, "      Walkthroughs");
            AddCategoryMapping(1545, TorznabCatType.Movies, "      Lineage II Movies");
            AddCategoryMapping(1546, TorznabCatType.Movies, "      World of Warcraft Movies");
            AddCategoryMapping(1547, TorznabCatType.Movies, "      Counter Strike Movies");
            AddCategoryMapping(1045, TorznabCatType.Movies, "   Video on moderation");
            AddCategoryMapping(1607, TorznabCatType.MoviesSD, "      DVD Video on moderation");
            AddCategoryMapping(1608, TorznabCatType.MoviesHD, "      HD Video on moderation");
            AddCategoryMapping(1837, TorznabCatType.PCGames, "Releases SE7ENKILLS");
            AddCategoryMapping(1839, TorznabCatType.PCGames, "   Games");
            AddCategoryMapping(1840, TorznabCatType.PCGames, "      Patches");
            AddCategoryMapping(1841, TorznabCatType.PCGames, "   Frequently asked questions about cs: go");
            AddCategoryMapping(1182, TorznabCatType.PCGames, "Games");
            AddCategoryMapping(158, TorznabCatType.PCGames, "   Games General Section");
            AddCategoryMapping(68, TorznabCatType.PCGames, "   Games for PC");
            AddCategoryMapping(69, TorznabCatType.PCGames, "      Hot New Releases Games");
            AddCategoryMapping(1030, TorznabCatType.PCGames, "         Games without pills");
            AddCategoryMapping(70, TorznabCatType.PCGames, "      Action");
            AddCategoryMapping(148, TorznabCatType.PCGames, "         FPS (1st Person)");
            AddCategoryMapping(149, TorznabCatType.PCGames, "         TPS (3rd Person)");
            AddCategoryMapping(150, TorznabCatType.PCGames, "         Stealth Action");
            AddCategoryMapping(151, TorznabCatType.PCGames, "         Tactical shooter");
            AddCategoryMapping(71, TorznabCatType.PCGames, "      RPG");
            AddCategoryMapping(72, TorznabCatType.PCGames, "      Strategy");
            AddCategoryMapping(152, TorznabCatType.PCGames, "         RTS (real time strategy)");
            AddCategoryMapping(153, TorznabCatType.PCGames, "         TBS (turn-based strategy)");
            AddCategoryMapping(154, TorznabCatType.PCGames, "         Wargame");
            AddCategoryMapping(155, TorznabCatType.PCGames, "         Economic strategies");
            AddCategoryMapping(73, TorznabCatType.PCGames, "      Simulations");
            AddCategoryMapping(74, TorznabCatType.PCGames, "         Autos and Racing");
            AddCategoryMapping(75, TorznabCatType.PCGames, "         Sports simulators");
            AddCategoryMapping(464, TorznabCatType.PCGames, "         Other simulators");
            AddCategoryMapping(1531, TorznabCatType.PCGames, "         Space and flight simulators");
            AddCategoryMapping(76, TorznabCatType.PCGames, "            Aviasimulators");
            AddCategoryMapping(463, TorznabCatType.PCGames, "            space Simulation");
            AddCategoryMapping(1540, TorznabCatType.PCGames, "            Microsoft Flight Simulator add-ons, and for him");
            AddCategoryMapping(1541, TorznabCatType.PCGames, "               Scripts, meshes and airports");
            AddCategoryMapping(1542, TorznabCatType.PCGames, "               Airplanes and helicopters");
            AddCategoryMapping(1543, TorznabCatType.PCGames, "               Mission, traffic sounds, packs and tools");
            AddCategoryMapping(1899, TorznabCatType.PCGames, "               Scenarios (FSX-P3D)");
            AddCategoryMapping(77, TorznabCatType.PCGames, "      Arcade");
            AddCategoryMapping(459, TorznabCatType.PCGames, "         Arcade (various)");
            AddCategoryMapping(461, TorznabCatType.PCGames, "         Board &amp; Card Arcade");
            AddCategoryMapping(78, TorznabCatType.PCGames, "      Adventure Quests");
            AddCategoryMapping(746, TorznabCatType.PCGames, "         Quest-style \"search objects\"");
            AddCategoryMapping(79, TorznabCatType.PCGames, "      Online Games");
            AddCategoryMapping(743, TorznabCatType.PCGames, "         Free");
            AddCategoryMapping(744, TorznabCatType.PCGames, "         paid");
            AddCategoryMapping(742, TorznabCatType.PCGames, "         Other online gaming");
            AddCategoryMapping(157, TorznabCatType.PCGames, "      For the little ones");
            AddCategoryMapping(465, TorznabCatType.PCGames, "      Old games for PC");
            AddCategoryMapping(466, TorznabCatType.PCGames, "         Arcade and Puzzle Games (old games)");
            AddCategoryMapping(1871, TorznabCatType.PCGames, "            Arcade (Old Games)");
            AddCategoryMapping(1872, TorznabCatType.PCGames, "            Puzzle games (old games)");
            AddCategoryMapping(467, TorznabCatType.PCGames, "         Adventure quests (old games)");
            AddCategoryMapping(468, TorznabCatType.PCGames, "         Action (old games)");
            AddCategoryMapping(469, TorznabCatType.PCGames, "         Strategy (old games)");
            AddCategoryMapping(470, TorznabCatType.PCGames, "         RPG (old games)");
            AddCategoryMapping(471, TorznabCatType.PCGames, "         Simulations (old games)");
            AddCategoryMapping(1532, TorznabCatType.PCGames, "            Autos and Racing (old games)");
            AddCategoryMapping(1533, TorznabCatType.PCGames, "            Space simulators, flight simulators and aviaigry (old games)");
            AddCategoryMapping(1534, TorznabCatType.PCGames, "            Sports simulators (old games)");
            AddCategoryMapping(1535, TorznabCatType.PCGames, "            Other simulators (Old Games)");
            AddCategoryMapping(472, TorznabCatType.PCGames, "         Multi-genre compilations (old games)");
            AddCategoryMapping(1536, TorznabCatType.PCGames, "         Erotic games (old games)");
            AddCategoryMapping(1537, TorznabCatType.PCGames, "         For the little ones (Old Games)");
            AddCategoryMapping(1538, TorznabCatType.PCGames, "         Puzzle Games (Old Games)");
            AddCategoryMapping(1539, TorznabCatType.PCGames, "         IBM PC are not compatible (old games)");
            AddCategoryMapping(473, TorznabCatType.PCGames, "      Erotic games");
            AddCategoryMapping(745, TorznabCatType.PCGames, "      Chess");
            AddCategoryMapping(924, TorznabCatType.PCGames, "      game Collections");
            AddCategoryMapping(970, TorznabCatType.PCGames, "      Other for PC-games");
            AddCategoryMapping(1803, TorznabCatType.PCGames, "         Patches");
            AddCategoryMapping(80, TorznabCatType.PCGames, "            Official patches");
            AddCategoryMapping(1790, TorznabCatType.PCGames, "         Fashion, plug-ins, add-ons");
            AddCategoryMapping(972, TorznabCatType.PCGames, "            Official mode, plug-ins, add-ons");
            AddCategoryMapping(162, TorznabCatType.PCGames, "            Informal fashion, plugins, add-ons");
            AddCategoryMapping(161, TorznabCatType.PCGames, "         Fun");
            AddCategoryMapping(973, TorznabCatType.PCGames, "         Editors, emulators and other gaming utility");
            AddCategoryMapping(160, TorznabCatType.PCGames, "         NoCD / NoDVD");
            AddCategoryMapping(974, TorznabCatType.PCGames, "         Conservation games");
            AddCategoryMapping(971, TorznabCatType.PCGames, "         Cheat program and trainers");
            AddCategoryMapping(164, TorznabCatType.PCGames, "         Guidelines and passing");
            AddCategoryMapping(163, TorznabCatType.PCGames, "         The bonus disc for the games");
            AddCategoryMapping(159, TorznabCatType.PCGames, "      The demo version of the game and with early access");
            AddCategoryMapping(975, TorznabCatType.PCGames, "      Anime Games");
            AddCategoryMapping(1025, TorznabCatType.PCGames, "      Fighting");
            AddCategoryMapping(460, TorznabCatType.PCGames, "      Logic games");
            AddCategoryMapping(462, TorznabCatType.PCGames, "      Mini / Flash games");
            AddCategoryMapping(1029, TorznabCatType.PCGames, "      Indie Game");
            AddCategoryMapping(111, TorznabCatType.Console, "   Games for consoles");
            AddCategoryMapping(458, TorznabCatType.Console, "      Portable and Console Games (general section of games for different platforms)");
            AddCategoryMapping(129, TorznabCatType.ConsoleXbox, "      Xbox");
            AddCategoryMapping(131, TorznabCatType.ConsoleXbox360, "         XBox360 | Games");
            AddCategoryMapping(132, TorznabCatType.ConsoleXbox360, "         XBox360 | GOD Games");
            AddCategoryMapping(133, TorznabCatType.ConsoleXbox360, "         XBox360 | JTAG");
            AddCategoryMapping(134, TorznabCatType.ConsoleXbox360, "         XBox360 | 360E");
            AddCategoryMapping(135, TorznabCatType.ConsoleXbox360, "         XBox360 | Demo");
            AddCategoryMapping(136, TorznabCatType.ConsoleXbox360, "         XBox360 | Soft");
            AddCategoryMapping(137, TorznabCatType.ConsoleXbox, "         Original XBox | Games");
            AddCategoryMapping(138, TorznabCatType.ConsolePS4, "      PlayStation");
            AddCategoryMapping(621, TorznabCatType.ConsolePS3, "         PS");
            AddCategoryMapping(141, TorznabCatType.ConsolePS3, "         PS2 | Games");
            AddCategoryMapping(112, TorznabCatType.ConsolePS3, "         PS3 | Games");
            AddCategoryMapping(142, TorznabCatType.ConsolePS3, "         PS3 | Other");
            AddCategoryMapping(139, TorznabCatType.ConsolePS4, "         PSN | Games");
            AddCategoryMapping(140, TorznabCatType.ConsolePSP, "         PSP | Games");
            AddCategoryMapping(622, TorznabCatType.ConsolePSP, "         PS1 games for PSP");
            AddCategoryMapping(143, TorznabCatType.ConsolePSP, "         PSP | Programs | Other");
            AddCategoryMapping(1548, TorznabCatType.ConsolePSP, "            Software for PSP (Homebrew)");
            AddCategoryMapping(455, TorznabCatType.ConsolePSVita, "         PS Vita | Games");
            AddCategoryMapping(130, TorznabCatType.ConsoleOther, "      Nintendo");
            AddCategoryMapping(144, TorznabCatType.ConsoleNDS, "         NDS | Games");
            AddCategoryMapping(145, TorznabCatType.ConsoleWii, "         Wii | Games");
            AddCategoryMapping(146, TorznabCatType.ConsoleWiiwareVC, "         WiiWare | Games");
            AddCategoryMapping(147, TorznabCatType.ConsoleOther, "         GameCube | Games");
            AddCategoryMapping(456, TorznabCatType.ConsoleOther, "      Sega");
            AddCategoryMapping(588, TorznabCatType.ConsoleOther, "         Dreamcast");
            AddCategoryMapping(457, TorznabCatType.ConsoleOther, "      Games for older consoles");
            AddCategoryMapping(589, TorznabCatType.ConsoleOther, "      Games for the DVD player");
            AddCategoryMapping(928, TorznabCatType.PCGames, "   Games for Linux");
            AddCategoryMapping(1868, TorznabCatType.PCGames, "      Native games for Linux");
            AddCategoryMapping(1869, TorznabCatType.PCGames, "      Ported games for Linux");
            AddCategoryMapping(1870, TorznabCatType.PCGames, "      Archive for Linux games");
            AddCategoryMapping(81, TorznabCatType.PC0day, "Software");
            AddCategoryMapping(570, TorznabCatType.PC0day, "   General Section for software");
            AddCategoryMapping(109, TorznabCatType.PCPhoneOther, "   Software for smart phones, mobile phones and PDAs");
            AddCategoryMapping(899, TorznabCatType.PCPhoneOther, "      Java");
            AddCategoryMapping(900, TorznabCatType.PCPhoneOther, "         Games for Java");
            AddCategoryMapping(901, TorznabCatType.PCPhoneOther, "         Applications for Java");
            AddCategoryMapping(590, TorznabCatType.PCPhoneAndroid, "      Android");
            AddCategoryMapping(592, TorznabCatType.PCPhoneAndroid, "         Games for Android");
            AddCategoryMapping(1017, TorznabCatType.PCPhoneAndroid, "            Games for Android [Eng]");
            AddCategoryMapping(895, TorznabCatType.PCPhoneAndroid, "         Apps for Android OS");
            AddCategoryMapping(1018, TorznabCatType.PCPhoneAndroid, "            Applications for Android [Eng]");
            AddCategoryMapping(480, TorznabCatType.PCPhoneIOS, "      Apple Mobile Device Software");
            AddCategoryMapping(481, TorznabCatType.PCPhoneIOS, "         Firmware (iPhone / iPod Touch / iPad / Apple TV)");
            AddCategoryMapping(482, TorznabCatType.PCPhoneIOS, "         Programs for iOS (iPhone / iPod Touch / iPad)");
            AddCategoryMapping(483, TorznabCatType.PCPhoneIOS, "         Games for iOS (iPhone / iPod Touch / iPad)");
            AddCategoryMapping(485, TorznabCatType.PCPhoneIOS, "         Miscellaneous iOS (iPhone / iPod Touch / iPad)");
            AddCategoryMapping(896, TorznabCatType.PCPhoneOther, "      Symbian");
            AddCategoryMapping(897, TorznabCatType.PCPhoneOther, "         Games for Symbian");
            AddCategoryMapping(898, TorznabCatType.PCPhoneOther, "         Applications for Symbian");
            AddCategoryMapping(902, TorznabCatType.PCPhoneOther, "      Windows Mobile, Palm OS, BlackBerry etc");
            AddCategoryMapping(903, TorznabCatType.PCPhoneOther, "         Games for Windows Mobile, Palm OS, BlackBerry etc");
            AddCategoryMapping(904, TorznabCatType.PCPhoneOther, "         Applications for Windows Mobile, Palm OS, BlackBerry etc");
            AddCategoryMapping(1579, TorznabCatType.PCPhoneOther, "      Windows Phone 7,8");
            AddCategoryMapping(1580, TorznabCatType.PCPhoneOther, "         Games for Windows Phone 7,8");
            AddCategoryMapping(1581, TorznabCatType.PCPhoneOther, "         Applications for Windows Phone 7,8");
            AddCategoryMapping(1582, TorznabCatType.PCPhoneOther, "      Software for your phone");
            AddCategoryMapping(1583, TorznabCatType.PCPhoneOther, "      Firmware for phones");
            AddCategoryMapping(106, TorznabCatType.PC0day, "   On Linux, Unix etc");
            AddCategoryMapping(282, TorznabCatType.PC0day, "      Operating Systems (Linux, Unix)");
            AddCategoryMapping(1574, TorznabCatType.PC0day, "      Program (Linux, Unix)");
            AddCategoryMapping(284, TorznabCatType.PC0day, "      Other operating systems and software for them");
            AddCategoryMapping(287, TorznabCatType.PC0day, "      Archive (Linux OS, Unix etc)");
            AddCategoryMapping(276, TorznabCatType.PCMac, "   Apple OS");
            AddCategoryMapping(277, TorznabCatType.PCMac, "      Mac OS [for Macintosh]");
            AddCategoryMapping(278, TorznabCatType.PCMac, "      Mac OS [PC-Hackintosh]");
            AddCategoryMapping(591, TorznabCatType.PCMac, "      Games Mac OS");
            AddCategoryMapping(1019, TorznabCatType.PCMac, "         Mac Games [ENG]");
            AddCategoryMapping(1021, TorznabCatType.PCMac, "      Program for viewing and video processing (Mac OS)");
            AddCategoryMapping(1022, TorznabCatType.PCMac, "      Programs for creating and processing graphs (Mac OS)");
            AddCategoryMapping(1023, TorznabCatType.PCMac, "      Plug-ins for Adobe's software (Mac OS)");
            AddCategoryMapping(1584, TorznabCatType.PCMac, "      Audio editor and converter (Mac OS)");
            AddCategoryMapping(1585, TorznabCatType.PCMac, "      System software (Mac OS)");
            AddCategoryMapping(1586, TorznabCatType.PCMac, "      Office software (Mac OS)");
            AddCategoryMapping(1587, TorznabCatType.PCMac, "      Programs for the Internet and network (Mac OS)");
            AddCategoryMapping(1588, TorznabCatType.PCMac, "      Other software (Mac OS)");
            AddCategoryMapping(103, TorznabCatType.PC0day, "   Microsoft OS");
            AddCategoryMapping(104, TorznabCatType.PC0day, "      Desktop operating system from Microsoft (released prior to Windows XP)");
            AddCategoryMapping(105, TorznabCatType.PC0day, "      Desktop operating system from Microsoft (since Windows XP)");
            AddCategoryMapping(1629, TorznabCatType.PC0day, "         Windows XP");
            AddCategoryMapping(1628, TorznabCatType.PC0day, "         Windows Vista");
            AddCategoryMapping(981, TorznabCatType.PC0day, "         Windows 7");
            AddCategoryMapping(1610, TorznabCatType.PC0day, "         Windows 8");
            AddCategoryMapping(1811, TorznabCatType.PC0day, "         Windows 10");
            AddCategoryMapping(274, TorznabCatType.PC0day, "      Windows Server");
            AddCategoryMapping(927, TorznabCatType.PC0day, "      Other (Operating Systems from Microsoft)");
            AddCategoryMapping(275, TorznabCatType.PC0day, "      Archive (OS from Microsoft)");
            AddCategoryMapping(84, TorznabCatType.PC0day, "   System programs");
            AddCategoryMapping(86, TorznabCatType.PC0day, "      Programs for configuring and optimizing OS");
            AddCategoryMapping(87, TorznabCatType.PC0day, "      Archivers and File Managers");
            AddCategoryMapping(1630, TorznabCatType.PC0day, "      Safety protection system and PC");
            AddCategoryMapping(93, TorznabCatType.PC0day, "         Software to protect your computer (antivirus software, firewalls)");
            AddCategoryMapping(580, TorznabCatType.PC0day, "            Keys and Activation");
            AddCategoryMapping(94, TorznabCatType.PC0day, "         Anti-spyware and anti-trojan");
            AddCategoryMapping(95, TorznabCatType.PC0day, "         Programs for the protection of information");
            AddCategoryMapping(88, TorznabCatType.PC0day, "      Backup");
            AddCategoryMapping(89, TorznabCatType.PC0day, "      Service computer service");
            AddCategoryMapping(1631, TorznabCatType.PC0day, "         LiveCD / DVD / Flash etc");
            AddCategoryMapping(90, TorznabCatType.PC0day, "      Work with data carriers");
            AddCategoryMapping(91, TorznabCatType.PC0day, "      Information and Diagnostics");
            AddCategoryMapping(92, TorznabCatType.PC0day, "      Programs for Internet and networks");
            AddCategoryMapping(96, TorznabCatType.PC0day, "      Drivers and Firmware");
            AddCategoryMapping(97, TorznabCatType.PC0day, "      Original disks to computers and accessories");
            AddCategoryMapping(98, TorznabCatType.PC0day, "      Server Software for Windows");
            AddCategoryMapping(99, TorznabCatType.PC0day, "      Change the Windows interface");
            AddCategoryMapping(101, TorznabCatType.PC0day, "      Screensavers");
            AddCategoryMapping(85, TorznabCatType.PC0day, "      Work with hard drive");
            AddCategoryMapping(102, TorznabCatType.PC0day, "      Miscellaneous (System programs on Windows)");
            AddCategoryMapping(82, TorznabCatType.PC0day, "   Systems for business, office, research and project work");
            AddCategoryMapping(83, TorznabCatType.PC0day, "      Business Systems");
            AddCategoryMapping(585, TorznabCatType.PC0day, "         The company's products 1C");
            AddCategoryMapping(1829, TorznabCatType.PC0day, "            typical configuration");
            AddCategoryMapping(1830, TorznabCatType.PC0day, "            industry configuration");
            AddCategoryMapping(1831, TorznabCatType.PC0day, "            ITS 2015");
            AddCategoryMapping(1832, TorznabCatType.PC0day, "            ITS 2014");
            AddCategoryMapping(1833, TorznabCatType.PC0day, "            ITS 2013");
            AddCategoryMapping(1834, TorznabCatType.PC0day, "            ITS 2012");
            AddCategoryMapping(1835, TorznabCatType.PC0day, "            ITS 2011");
            AddCategoryMapping(1836, TorznabCatType.PC0day, "            Archive (Products 1C)");
            AddCategoryMapping(270, TorznabCatType.PC0day, "      Office systems");
            AddCategoryMapping(266, TorznabCatType.PC0day, "      Dictionaries, translators");
            AddCategoryMapping(272, TorznabCatType.PC0day, "      Recognition of text, sound and speech synthesis");
            AddCategoryMapping(269, TorznabCatType.PC0day, "      Miscellaneous (business systems, office, research and project work)");
            AddCategoryMapping(302, TorznabCatType.PC0day, "      CAD software for architects");
            AddCategoryMapping(593, TorznabCatType.PC0day, "         CAD (general and engineering)");
            AddCategoryMapping(594, TorznabCatType.PC0day, "         CAD (electronics, automation, GAP)");
            AddCategoryMapping(940, TorznabCatType.PC0day, "         Programs for architects and builders");
            AddCategoryMapping(937, TorznabCatType.PC0day, "      All for house: dressmaking, sewing, cooking");
            AddCategoryMapping(938, TorznabCatType.PC0day, "      Work with PDF and DjVu");
            AddCategoryMapping(939, TorznabCatType.PC0day, "      Systems for scientific work");
            AddCategoryMapping(941, TorznabCatType.PC0day, "      Libraries and projects for architects and interior designers");
            AddCategoryMapping(942, TorznabCatType.PC0day, "      Other reference systems");
            AddCategoryMapping(107, TorznabCatType.PC0day, "   Web Development and Programming");
            AddCategoryMapping(293, TorznabCatType.PC0day, "      Search / Offer");
            AddCategoryMapping(943, TorznabCatType.PC0day, "      WYSIWYG editors for web design");
            AddCategoryMapping(496, TorznabCatType.PC0day, "      Database Management Systems (DBMS)");
            AddCategoryMapping(494, TorznabCatType.PC0day, "      programming environments, compilers and software tools");
            AddCategoryMapping(290, TorznabCatType.PC0day, "      The components for the development of media");
            AddCategoryMapping(495, TorznabCatType.PC0day, "      Text editors Illuminated");
            AddCategoryMapping(291, TorznabCatType.PC0day, "      Scripting engines and websites, CMS and extensions to it");
            AddCategoryMapping(944, TorznabCatType.PC0day, "      Templates for websites and CMS");
            AddCategoryMapping(292, TorznabCatType.PC0day, "      Miscellaneous (Web Development and Programming)");
            AddCategoryMapping(294, TorznabCatType.PC0day, "      Archive (Web Development and Programming)");
            AddCategoryMapping(108, TorznabCatType.PC0day, "   Programs to work with multimedia and 3D");
            AddCategoryMapping(487, TorznabCatType.PC0day, "      Software kits");
            AddCategoryMapping(488, TorznabCatType.PC0day, "      Plug-ins for Adobe's programs");
            AddCategoryMapping(491, TorznabCatType.PC0day, "      3D modeling, rendering and plugins for them");
            AddCategoryMapping(489, TorznabCatType.PC0day, "      Graphic editor");
            AddCategoryMapping(303, TorznabCatType.PC0day, "      Editors video");
            AddCategoryMapping(305, TorznabCatType.PC0day, "      Virtual Studios, sequencers and audio editor");
            AddCategoryMapping(492, TorznabCatType.PC0day, "      Animation");
            AddCategoryMapping(490, TorznabCatType.PC0day, "      Programs for typesetting, printing, and working with fonts");
            AddCategoryMapping(304, TorznabCatType.PC0day, "      Video, Audio Conversion");
            AddCategoryMapping(493, TorznabCatType.PC0day, "      Creating a BD / HD / DVD-Video");
            AddCategoryMapping(306, TorznabCatType.PC0day, "      Plug-ins for sound processing");
            AddCategoryMapping(308, TorznabCatType.PC0day, "      Archive (Programme for multimedia and 3D)");
            AddCategoryMapping(595, TorznabCatType.PC0day, "      Miscellaneous (Programme for multimedia and 3D)");
            AddCategoryMapping(596, TorznabCatType.PC0day, "      Miscellaneous (Programs for working with audio)");
            AddCategoryMapping(597, TorznabCatType.PC0day, "      Virtual instruments and synthesizers");
            AddCategoryMapping(264, TorznabCatType.PC0day, "      Audio and video, players and catalogers");
            AddCategoryMapping(263, TorznabCatType.PC0day, "      Cataloging and graphics viewers");
            AddCategoryMapping(348, TorznabCatType.PC0day, "   Materials for Multimedia and Design");
            AddCategoryMapping(945, TorznabCatType.PC0day, "      Author's works");
            AddCategoryMapping(946, TorznabCatType.PC0day, "      Official collection vector clipart");
            AddCategoryMapping(948, TorznabCatType.PC0day, "      Other vector cliparts");
            AddCategoryMapping(949, TorznabCatType.PC0day, "      Photostosks");
            AddCategoryMapping(950, TorznabCatType.PC0day, "      The costumes for the photomontage");
            AddCategoryMapping(350, TorznabCatType.PC0day, "      Frames and Vignettes for processing photos");
            AddCategoryMapping(352, TorznabCatType.PC0day, "      Other raster clipart");
            AddCategoryMapping(1634, TorznabCatType.PC0day, "         backgrounds");
            AddCategoryMapping(1635, TorznabCatType.PC0day, "         Templates");
            AddCategoryMapping(1637, TorznabCatType.PC0day, "         Raster Graphics (photos)");
            AddCategoryMapping(1638, TorznabCatType.PC0day, "         Raster Graphics (elements)");
            AddCategoryMapping(1639, TorznabCatType.PC0day, "         Raster Graphics (illustrations)");
            AddCategoryMapping(353, TorznabCatType.PC0day, "      3D models, scenes and materials");
            AddCategoryMapping(1633, TorznabCatType.PC0day, "         Textures");
            AddCategoryMapping(354, TorznabCatType.PC0day, "      Footage");
            AddCategoryMapping(951, TorznabCatType.PC0day, "      Other collections footage");
            AddCategoryMapping(952, TorznabCatType.PC0day, "      music library");
            AddCategoryMapping(355, TorznabCatType.PC0day, "      Sound effects");
            AddCategoryMapping(953, TorznabCatType.PC0day, "      sample Libraries");
            AddCategoryMapping(954, TorznabCatType.PC0day, "      Libraries and saundbanki for samplers, synth presets");
            AddCategoryMapping(955, TorznabCatType.PC0day, "      Multitracks");
            AddCategoryMapping(956, TorznabCatType.PC0day, "      Materials for creating menus and DVD covers");
            AddCategoryMapping(351, TorznabCatType.PC0day, "      Styles, brushes, shapes and patterns for Adobe Photoshop");
            AddCategoryMapping(356, TorznabCatType.PC0day, "      Fonts");
            AddCategoryMapping(358, TorznabCatType.PC0day, "      Miscellaneous (Materials for Multimedia and Design)");
            AddCategoryMapping(1632, TorznabCatType.PC0day, "      Digital Juice");
            AddCategoryMapping(1874, TorznabCatType.PC0day, "      Projects");
            AddCategoryMapping(1875, TorznabCatType.PC0day, "         Children (projects)");
            AddCategoryMapping(1876, TorznabCatType.PC0day, "         Wedding and romantic (projects)");
            AddCategoryMapping(1877, TorznabCatType.PC0day, "         Holiday (projects)");
            AddCategoryMapping(1878, TorznabCatType.PC0day, "         Presentations (projects)");
            AddCategoryMapping(1879, TorznabCatType.PC0day, "         Sport (projects)");
            AddCategoryMapping(1880, TorznabCatType.PC0day, "         Logos (projects)");
            AddCategoryMapping(1881, TorznabCatType.PC0day, "         Slideshow (projects)");
            AddCategoryMapping(1882, TorznabCatType.PC0day, "         Titles (projects)");
            AddCategoryMapping(1883, TorznabCatType.PC0day, "         Items (Projects)");
            AddCategoryMapping(1884, TorznabCatType.PC0day, "         Miscellaneous (projects)");
            AddCategoryMapping(1898, TorznabCatType.PC0day, "         Trailers (projects)");
            AddCategoryMapping(295, TorznabCatType.PC0day, "   Reference and legal system");
            AddCategoryMapping(296, TorznabCatType.PC0day, "      Consultant Plus");
            AddCategoryMapping(584, TorznabCatType.PC0day, "         Consultant Accountant");
            AddCategoryMapping(755, TorznabCatType.PC0day, "         Archive irrelevant hands");
            AddCategoryMapping(297, TorznabCatType.PC0day, "      code");
            AddCategoryMapping(298, TorznabCatType.PC0day, "      Guarantee");
            AddCategoryMapping(299, TorznabCatType.PC0day, "      other");
            AddCategoryMapping(300, TorznabCatType.PC0day, "      Archive (Reference and legal system)");
            AddCategoryMapping(301, TorznabCatType.PC0day, "      RG PCA - a hidden forum");
            AddCategoryMapping(587, TorznabCatType.PC0day, "   Collections of programs and WPI");
            AddCategoryMapping(929, TorznabCatType.PC0day, "   Test drives to adjust the audio / video equipment");
            AddCategoryMapping(957, TorznabCatType.PC0day, "   GIS, navigation systems and maps");
            AddCategoryMapping(958, TorznabCatType.PC0day, "      GIS (Geographic Information Systems)");
            AddCategoryMapping(959, TorznabCatType.PC0day, "      Maps provided with the program shell");
            AddCategoryMapping(960, TorznabCatType.PC0day, "      Atlases and maps modern (after 1950)");
            AddCategoryMapping(961, TorznabCatType.PC0day, "      Atlases and maps of old (pre-1950)");
            AddCategoryMapping(962, TorznabCatType.PC0day, "      Cards Other (astronomical, historical, thematic)");
            AddCategoryMapping(963, TorznabCatType.PC0day, "      Built-in car navigation");
            AddCategoryMapping(964, TorznabCatType.PC0day, "      Garmin");
            AddCategoryMapping(965, TorznabCatType.PC0day, "      Ozi");
            AddCategoryMapping(966, TorznabCatType.PC0day, "      TomTom");
            AddCategoryMapping(967, TorznabCatType.PC0day, "      Navigon / Navitel");
            AddCategoryMapping(968, TorznabCatType.PC0day, "      Igo");
            AddCategoryMapping(969, TorznabCatType.PC0day, "      Miscellaneous - navigation and maps");
            AddCategoryMapping(61, TorznabCatType.Audio, "Music and music videos");
            AddCategoryMapping(579, TorznabCatType.Audio, "   Total music section");
            AddCategoryMapping(537, TorznabCatType.AudioVideo, "      General Discussion unsorted music video");
            AddCategoryMapping(538, TorznabCatType.AudioVideo, "         Music video (HD / DVD)");
            AddCategoryMapping(544, TorznabCatType.AudioVideo, "         Concert recording");
            AddCategoryMapping(1781, TorznabCatType.AudioVideo, "            Concerts (DVD)");
            AddCategoryMapping(1782, TorznabCatType.AudioVideo, "            Concerts (HD)");
            AddCategoryMapping(1784, TorznabCatType.AudioVideo, "         Opera, Ballet, Musicals");
            AddCategoryMapping(1785, TorznabCatType.AudioVideo, "         Music videos (transit)");
            AddCategoryMapping(501, TorznabCatType.AudioLossless, "      unsorted Lossless");
            AddCategoryMapping(532, TorznabCatType.Audio, "      Multi-channel HD Audio and Music");
            AddCategoryMapping(533, TorznabCatType.Audio, "         DVD-Audio, SACD, Audio-DVD");
            AddCategoryMapping(1687, TorznabCatType.Audio, "            DVD-Audio");
            AddCategoryMapping(1688, TorznabCatType.Audio, "            SACD-R");
            AddCategoryMapping(534, TorznabCatType.Audio, "         DTS");
            AddCategoryMapping(535, TorznabCatType.Audio, "         Vinyl-Rip and Hand-Made");
            AddCategoryMapping(536, TorznabCatType.Audio, "         Hi-Res stereo");
            AddCategoryMapping(529, TorznabCatType.Audio, "      discography");
            AddCategoryMapping(530, TorznabCatType.Audio, "         Domestic");
            AddCategoryMapping(531, TorznabCatType.Audio, "         foreign");
            AddCategoryMapping(1679, TorznabCatType.Audio, "      Jazz, Blues, Soul (transit)");
            AddCategoryMapping(1680, TorznabCatType.Audio, "         Jazz (transit lossless)");
            AddCategoryMapping(1681, TorznabCatType.Audio, "         Jazz (transit lossy)");
            AddCategoryMapping(1682, TorznabCatType.Audio, "         Blues, Soul (transit lossless)");
            AddCategoryMapping(1683, TorznabCatType.Audio, "         Blues, Soul (transit lossy)");
            AddCategoryMapping(525, TorznabCatType.Audio, "         Jazz, Blues, Soul (transit lossless)");
            AddCategoryMapping(1689, TorznabCatType.Audio, "      Rock, Alternative, Punk, Metal (transit)");
            AddCategoryMapping(521, TorznabCatType.Audio, "         Rock (transit Lossless)");
            AddCategoryMapping(1691, TorznabCatType.Audio, "         Rock (transit Lossy)");
            AddCategoryMapping(1692, TorznabCatType.Audio, "         Alternative, Punk (transit Lossless)");
            AddCategoryMapping(1693, TorznabCatType.Audio, "         Alternative, Punk (transit Lossy)");
            AddCategoryMapping(1694, TorznabCatType.Audio, "         Hard Rock (transit Lossless)");
            AddCategoryMapping(1695, TorznabCatType.Audio, "         Hard Rock (transit Lossy)");
            AddCategoryMapping(506, TorznabCatType.Audio, "         Metal (transit Lossless)");
            AddCategoryMapping(1697, TorznabCatType.Audio, "         Metal (transit Lossy)");
            AddCategoryMapping(1698, TorznabCatType.Audio, "         Russian Rock (transit Lossless)");
            AddCategoryMapping(1699, TorznabCatType.Audio, "         Russian Rock (transit Lossy)");
            AddCategoryMapping(1817, TorznabCatType.Audio, "         Rock, Alternative, Punk, Metal (transit Lossless)");
            AddCategoryMapping(1700, TorznabCatType.Audio, "      Popular music (transit)");
            AddCategoryMapping(1701, TorznabCatType.Audio, "         Eurodance, Euro-House, Technopop (transit Lossless)");
            AddCategoryMapping(1702, TorznabCatType.Audio, "         Eurodance, Euro-House, Technopop (transit Lossy)");
            AddCategoryMapping(1703, TorznabCatType.Audio, "         Disco, Italo-Disco, Euro-Disco, Hi-NRG (transit Lossless)");
            AddCategoryMapping(1704, TorznabCatType.Audio, "         Disco, Italo-Disco, Euro-Disco, Hi-NRG (transit Lossy)");
            AddCategoryMapping(1705, TorznabCatType.Audio, "         Patriotic Pop (transit lossless)");
            AddCategoryMapping(1706, TorznabCatType.Audio, "         Patriotic Pop (transit Lossy)");
            AddCategoryMapping(1707, TorznabCatType.Audio, "         Soviet stage, Retro (transit Lossless)");
            AddCategoryMapping(1708, TorznabCatType.Audio, "         Soviet stage, Retro (transit Lossy)");
            AddCategoryMapping(1709, TorznabCatType.Audio, "         Foreign pop music (transit Lossless)");
            AddCategoryMapping(1710, TorznabCatType.Audio, "         Foreign pop music (transit Lossy)");
            AddCategoryMapping(1818, TorznabCatType.Audio, "         Popular music (transit Lossless)");
            AddCategoryMapping(1711, TorznabCatType.Audio, "      Electronic music (transit)");
            AddCategoryMapping(1712, TorznabCatType.Audio, "         Trance (transit Lossless)");
            AddCategoryMapping(1713, TorznabCatType.Audio, "         Trance (transit Lossy)");
            AddCategoryMapping(1714, TorznabCatType.Audio, "         House, Techno, Electro, Minimal (transit Lossless)");
            AddCategoryMapping(1715, TorznabCatType.Audio, "         House (transit Lossy)");
            AddCategoryMapping(1716, TorznabCatType.Audio, "         Techno, Electro, Minimal (transit Lossy)");
            AddCategoryMapping(1717, TorznabCatType.Audio, "         Easy listening (transit Lossless)");
            AddCategoryMapping(1718, TorznabCatType.Audio, "         Easy listening (transit Lossy)");
            AddCategoryMapping(1719, TorznabCatType.Audio, "         Experimental, Industrial, EBM, Dark Electro (Lossless)");
            AddCategoryMapping(1720, TorznabCatType.Audio, "         Experimental Electronic (transit Lossy)");
            AddCategoryMapping(1721, TorznabCatType.Audio, "         Industrial, EBM, Dark Electro (transit Lossy)");
            AddCategoryMapping(1722, TorznabCatType.Audio, "         Synthpop, New Wave (Lossless transit)");
            AddCategoryMapping(1723, TorznabCatType.Audio, "         Synthpop, New Wave (transit Lossy)");
            AddCategoryMapping(1724, TorznabCatType.Audio, "         Drum'n'Bass, Jungle, Breaks, Breakbeat, Dubstep (transit Lossless)");
            AddCategoryMapping(1725, TorznabCatType.Audio, "         Drum'n'Bass, Jungle, Breaks, Breakbeat, Dubstep (transit Lossy)");
            AddCategoryMapping(1726, TorznabCatType.Audio, "         Hardstyle, Jumpstyle, Hardcore (transit Lossless)");
            AddCategoryMapping(1727, TorznabCatType.Audio, "         Hardstyle, Jumpstyle, Hardcore (transit Lossy)");
            AddCategoryMapping(1728, TorznabCatType.Audio, "         Psychedelic, psytrance, fullon (transit Lossless)");
            AddCategoryMapping(1729, TorznabCatType.Audio, "         Psychedelic, psytrance, fullon (transit Lossy)");
            AddCategoryMapping(1730, TorznabCatType.Audio, "         Radioshow, Live Mixes (transit Lossy)");
            AddCategoryMapping(1820, TorznabCatType.Audio, "         Electronic music (transit Lossless)");
            AddCategoryMapping(1731, TorznabCatType.Audio, "      Rap, Hip-hop, RnB, Reggae (transit)");
            AddCategoryMapping(1732, TorznabCatType.Audio, "         Rap, Hip-hop overseas (transit Lossless)");
            AddCategoryMapping(1733, TorznabCatType.Audio, "         Rap, Hip-hop overseas (transit Lossy)");
            AddCategoryMapping(1734, TorznabCatType.Audio, "         Rap, Hip-hop domestic (Lossless)");
            AddCategoryMapping(1735, TorznabCatType.Audio, "         Rap, Hip-hop domestic (transit Lossy)");
            AddCategoryMapping(1736, TorznabCatType.Audio, "         RnB, Reggae (transit Lossless)");
            AddCategoryMapping(1737, TorznabCatType.Audio, "         RnB, Reggae (transit Lossy)");
            AddCategoryMapping(1819, TorznabCatType.Audio, "         Rap, Hip-hop, RnB (transit Lossless)");
            AddCategoryMapping(1738, TorznabCatType.Audio, "      East Asian Music (transit)");
            AddCategoryMapping(1739, TorznabCatType.Audio, "         Asian Traditional, Ethnic (transit Lossless)");
            AddCategoryMapping(1740, TorznabCatType.Audio, "         Asian Traditional, Ethnic (transit Lossy)");
            AddCategoryMapping(1741, TorznabCatType.Audio, "         Asian Pop (transit Lossless)");
            AddCategoryMapping(1742, TorznabCatType.Audio, "         Asian Pop (transit Lossy)");
            AddCategoryMapping(1743, TorznabCatType.Audio, "         Asian Rock, Metal (transit Lossless)");
            AddCategoryMapping(1744, TorznabCatType.Audio, "         Asian Rock, Metal (transit Lossy)");
            AddCategoryMapping(1745, TorznabCatType.Audio, "         Doujin Music (transit Lossless)");
            AddCategoryMapping(1746, TorznabCatType.Audio, "         Doujin Music (transit Lossy)");
            AddCategoryMapping(1747, TorznabCatType.Audio, "         Other Asian (transit Lossless)");
            AddCategoryMapping(1748, TorznabCatType.Audio, "         Other Asian (transit Lossy)");
            AddCategoryMapping(1749, TorznabCatType.Audio, "      Other Styles (transit)");
            AddCategoryMapping(1750, TorznabCatType.Audio, "         Instrumental (transit Lossless)");
            AddCategoryMapping(1751, TorznabCatType.Audio, "         Instrumental (transit Lossy)");
            AddCategoryMapping(1752, TorznabCatType.Audio, "         New Age / Meditative / Relax (transit Lossless)");
            AddCategoryMapping(1753, TorznabCatType.Audio, "         New Age / Meditative / Relax (transit Lossy)");
            AddCategoryMapping(1754, TorznabCatType.Audio, "         Classical Crossover / Neoclassical (transit Lossless)");
            AddCategoryMapping(1755, TorznabCatType.Audio, "         Classical Crossover / Neoclassical (transit Lossy)");
            AddCategoryMapping(1756, TorznabCatType.Audio, "         Folk (transit Lossless)");
            AddCategoryMapping(1757, TorznabCatType.Audio, "         Folk (transit Lossy)");
            AddCategoryMapping(1758, TorznabCatType.Audio, "         Other (transit Lossless)");
            AddCategoryMapping(1759, TorznabCatType.Audio, "         Other (transit Lossy)");
            AddCategoryMapping(1821, TorznabCatType.Audio, "            Folklore / Folk / Folk / World Music (transit Lossy)");
            AddCategoryMapping(1822, TorznabCatType.Audio, "            Compilations Folklore / Folk / Folk / World Music (transit Lossy)");
            AddCategoryMapping(1823, TorznabCatType.Audio, "            Country &amp; Folk (transit Lossy)");
            AddCategoryMapping(1760, TorznabCatType.Audio, "         OST (transit Lossless)");
            AddCategoryMapping(1761, TorznabCatType.Audio, "         OST (transit Lossy)");
            AddCategoryMapping(1762, TorznabCatType.Audio, "      Classic (transit)");
            AddCategoryMapping(1763, TorznabCatType.Audio, "         Notes (transit)");
            AddCategoryMapping(1764, TorznabCatType.Audio, "         Complete Works (transit)");
            AddCategoryMapping(1765, TorznabCatType.Audio, "         Vocals (transit)");
            AddCategoryMapping(1766, TorznabCatType.Audio, "         Concerts (transit)");
            AddCategoryMapping(1767, TorznabCatType.Audio, "         Orchestral (transit)");
            AddCategoryMapping(1768, TorznabCatType.Audio, "         Chamber (transit)");
            AddCategoryMapping(1769, TorznabCatType.Audio, "         Piano (transit)");
            AddCategoryMapping(1770, TorznabCatType.Audio, "         Compilations (transit)");
            AddCategoryMapping(1771, TorznabCatType.Audio, "         In processing (transit)");
            AddCategoryMapping(1783, TorznabCatType.Audio, "         Classic (transit Lossless)");
            AddCategoryMapping(1800, TorznabCatType.Audio, "      Author and military songs");
            AddCategoryMapping(1798, TorznabCatType.Audio, "         Author and war songs (transit Lossless)");
            AddCategoryMapping(1799, TorznabCatType.Audio, "         Author and war songs (transit Lossy)");
            AddCategoryMapping(1772, TorznabCatType.Audio, "      Unofficial collections (transit)");
            AddCategoryMapping(1773, TorznabCatType.Audio, "         Jazz, Blues, Soul (transit collections)");
            AddCategoryMapping(1774, TorznabCatType.Audio, "         Chanson, Author and war songs (collections transit)");
            AddCategoryMapping(1775, TorznabCatType.Audio, "         Rock, Alternative, Punk, Metal (collections of transit)");
            AddCategoryMapping(1776, TorznabCatType.Audio, "         Pop (transit collections)");
            AddCategoryMapping(1777, TorznabCatType.Audio, "         Electronic (transit collections)");
            AddCategoryMapping(1778, TorznabCatType.Audio, "         Rap, Hip-hop, RnB, Reggae (transit collections)");
            AddCategoryMapping(1779, TorznabCatType.Audio, "         Instrumental / New Age / Meditative / Relax (transit collections)");
            AddCategoryMapping(1780, TorznabCatType.Audio, "         Other (transit collections)");
            AddCategoryMapping(1816, TorznabCatType.Audio, "      Chanson, Author and war songs (transit)");
            AddCategoryMapping(1677, TorznabCatType.Audio, "         Chanson, Author and war songs (transit lossless)");
            AddCategoryMapping(1678, TorznabCatType.Audio, "         Chanson, Author and war songs (transit lossy)");
            AddCategoryMapping(63, TorznabCatType.Audio, "   Electonic music");
            AddCategoryMapping(784, TorznabCatType.Audio, "      Chillout, Lounge, Downtempo, Trip-Hop");
            AddCategoryMapping(785, TorznabCatType.Audio, "         Chillout, Lounge, Downtempo (lossless)");
            AddCategoryMapping(225, TorznabCatType.Audio, "         Chillout, Lounge, Downtempo (lossy)");
            AddCategoryMapping(786, TorznabCatType.Audio, "         Nu Jazz, Acid Jazz, Future Jazz (lossless)");
            AddCategoryMapping(787, TorznabCatType.Audio, "         Nu Jazz, Acid Jazz, Future Jazz (lossy)");
            AddCategoryMapping(788, TorznabCatType.Audio, "         Trip Hop, Abstract Hip-Hop (lossless)");
            AddCategoryMapping(789, TorznabCatType.Audio, "         Trip Hop, Abstract Hip-Hop (lossy)");
            AddCategoryMapping(800, TorznabCatType.Audio, "      Drum &amp; Bass, Jungle, Breakbeat, Dubstep, IDM, Electro");
            AddCategoryMapping(801, TorznabCatType.Audio, "         Dubstep (lossy)");
            AddCategoryMapping(224, TorznabCatType.Audio, "         Drum &amp; Bass, Jungle (lossy)");
            AddCategoryMapping(803, TorznabCatType.Audio, "         Drum &amp; Bass, Jungle (lossless)");
            AddCategoryMapping(804, TorznabCatType.Audio, "         Drum &amp; Bass, Jungle (Radioshows, Podcasts, Livesets, Mixes)");
            AddCategoryMapping(805, TorznabCatType.Audio, "         Breakbeat (lossless)");
            AddCategoryMapping(806, TorznabCatType.Audio, "         Breakbeat (lossy)");
            AddCategoryMapping(1517, TorznabCatType.Audio, "         Electro, Electro-Freestyle, Nu Electro (lossless)");
            AddCategoryMapping(1518, TorznabCatType.Audio, "         Electro, Electro-Freestyle, Nu Electro (lossy)");
            AddCategoryMapping(1519, TorznabCatType.Audio, "         Dubstep (lossless)");
            AddCategoryMapping(1520, TorznabCatType.Audio, "         Breakbeat, Dubstep (Radioshows, Podcasts, Livesets, Mixes)");
            AddCategoryMapping(1521, TorznabCatType.Audio, "         IDM (lossless)");
            AddCategoryMapping(1522, TorznabCatType.Audio, "         IDM (lossy)");
            AddCategoryMapping(1523, TorznabCatType.Audio, "         IDM Discography &amp; Collections (lossy)");
            AddCategoryMapping(227, TorznabCatType.Audio, "      Industrial, Noise, EBM, Dark Electro, Aggrotech, Synthpop, New Wave");
            AddCategoryMapping(774, TorznabCatType.Audio, "         EBM, Dark Electro, Aggrotech (lossless)");
            AddCategoryMapping(775, TorznabCatType.Audio, "         EBM, Dark Electro, Aggrotech (lossy)");
            AddCategoryMapping(776, TorznabCatType.Audio, "         Industrial, Noise (lossless)");
            AddCategoryMapping(777, TorznabCatType.Audio, "         Industrial, Noise (lossy)");
            AddCategoryMapping(778, TorznabCatType.Audio, "         Synthpop, New Wave (lossless)");
            AddCategoryMapping(779, TorznabCatType.Audio, "         Synthpop, New Wave (lossy)");
            AddCategoryMapping(780, TorznabCatType.Audio, "         Darkwave, Neoclassical, Ethereal, Dungeon Synth (lossless)");
            AddCategoryMapping(781, TorznabCatType.Audio, "         Darkwave, Neoclassical, Ethereal, Dungeon Synth (lossy)");
            AddCategoryMapping(752, TorznabCatType.Audio, "      House, Techno, Hardcore, Hardstyle, Jumpstyle");
            AddCategoryMapping(760, TorznabCatType.Audio, "         Hardcore, Hardstyle, Jumpstyle (lossy)");
            AddCategoryMapping(759, TorznabCatType.Audio, "         Hardcore, Hardstyle, Jumpstyle (lossless)");
            AddCategoryMapping(753, TorznabCatType.Audio, "         Hardcore, Hardstyle, Jumpstyle (vinyl, web)");
            AddCategoryMapping(223, TorznabCatType.Audio, "         House (lossy)");
            AddCategoryMapping(756, TorznabCatType.Audio, "         House (lossless)");
            AddCategoryMapping(770, TorznabCatType.Audio, "         House (Radioshow, Podcast, Liveset, Mixes)");
            AddCategoryMapping(754, TorznabCatType.Audio, "         House (Singles, EPs) (lossy)");
            AddCategoryMapping(771, TorznabCatType.Audio, "         House (Promorelizy, collections)");
            AddCategoryMapping(757, TorznabCatType.Audio, "         Techno (lossy)");
            AddCategoryMapping(758, TorznabCatType.Audio, "         Techno (lossless)");
            AddCategoryMapping(769, TorznabCatType.Audio, "         Techno (Radioshows, Podcasts, Livesets, Mixes)");
            AddCategoryMapping(761, TorznabCatType.Audio, "         Techno (Singles, EPs) (lossy)");
            AddCategoryMapping(790, TorznabCatType.Audio, "      Trance, Goa Trance, Psy-Trance, PsyChill, Ambient Dub");
            AddCategoryMapping(792, TorznabCatType.Audio, "         Goa Trance, Psy-Trance (lossless)");
            AddCategoryMapping(793, TorznabCatType.Audio, "         Goa Trance, Psy-Trance (lossy)");
            AddCategoryMapping(798, TorznabCatType.Audio, "         Goa / Psy-Trance, PsyChill and Ambient Dub (Radioshows, Livesets, Mixes) (lossy)");
            AddCategoryMapping(791, TorznabCatType.Audio, "         Trance (lossless)");
            AddCategoryMapping(222, TorznabCatType.Audio, "         Trance (lossy)");
            AddCategoryMapping(795, TorznabCatType.Audio, "         Trance (Radioshows, Podcasts, Live Sets, Mixes) (lossy)");
            AddCategoryMapping(794, TorznabCatType.Audio, "         Trance (Singles, EPs) (lossy)");
            AddCategoryMapping(796, TorznabCatType.Audio, "         PsyChill, Ambient Dub (lossless)");
            AddCategoryMapping(797, TorznabCatType.Audio, "         PsyChill, Ambient Dub (lossy)");
            AddCategoryMapping(762, TorznabCatType.Audio, "      Traditional Electronic, Ambient, Modern Classical, Electroacoustic, Experimental");
            AddCategoryMapping(768, TorznabCatType.Audio, "         8-bit, Chiptune (lossy &amp; lossless)");
            AddCategoryMapping(679, TorznabCatType.Audio, "         Experimental (lossy)");
            AddCategoryMapping(764, TorznabCatType.Audio, "         Experimental (lossless)");
            AddCategoryMapping(767, TorznabCatType.Audio, "         Modern Classical, Electroacoustic (lossy)");
            AddCategoryMapping(766, TorznabCatType.Audio, "         Modern Classical, Electroacoustic (lossless)");
            AddCategoryMapping(226, TorznabCatType.Audio, "         Traditional Electronic, Ambient (lossy)");
            AddCategoryMapping(763, TorznabCatType.Audio, "         Traditional Electronic, Ambient (lossless)");
            AddCategoryMapping(799, TorznabCatType.Audio, "      Label Packs (lossless)");
            AddCategoryMapping(802, TorznabCatType.Audio, "      Label packs, Scene packs (lossy)");
            AddCategoryMapping(782, TorznabCatType.Audio, "      Electronic music (Video, DVD Video / Audio, HD Video, DTS, SACD)");
            AddCategoryMapping(783, TorznabCatType.Audio, "         Electronic music (own digitization)");
            AddCategoryMapping(914, TorznabCatType.Audio, "         Electronic music (Video)");
            AddCategoryMapping(1524, TorznabCatType.Audio, "         Electronic music (Official DVD Video)");
            AddCategoryMapping(1525, TorznabCatType.Audio, "         Electronic music (Informal amateur DVD Video)");
            AddCategoryMapping(1526, TorznabCatType.Audio, "         Electronic Music (Hi-Res stereo)");
            AddCategoryMapping(1527, TorznabCatType.Audio, "         Multichannel music (Electronics)");
            AddCategoryMapping(1528, TorznabCatType.Audio, "         Electronic music (HD Video)");
            AddCategoryMapping(64, TorznabCatType.Audio, "   Rap, Hip-Hop, R'n'B");
            AddCategoryMapping(213, TorznabCatType.Audio, "      Foreign Rap, Hip-Hop (lossy)");
            AddCategoryMapping(214, TorznabCatType.Audio, "      Domestic Rap, Hip-Hop (lossy)");
            AddCategoryMapping(692, TorznabCatType.Audio, "      Foreign R'n'B (lossy)");
            AddCategoryMapping(894, TorznabCatType.Audio, "      Foreign Rap, Hip-Hop (lossless)");
            AddCategoryMapping(1387, TorznabCatType.Audio, "      Minus (Instrumentals)");
            AddCategoryMapping(1388, TorznabCatType.Audio, "      Domestic R'n'B (lossy)");
            AddCategoryMapping(1389, TorznabCatType.Audio, "      Domestic Rap, Hip-Hop, R'n'B (lossless)");
            AddCategoryMapping(1390, TorznabCatType.Audio, "      Domestic Rap, Hip-Hop (Video)");
            AddCategoryMapping(1391, TorznabCatType.Audio, "      Domestic R'n'B (Video)");
            AddCategoryMapping(1392, TorznabCatType.Audio, "      Foreign R'n'B (lossless)");
            AddCategoryMapping(1393, TorznabCatType.Audio, "      Rap, Hip-Hop, R'n'B (own digitization)");
            AddCategoryMapping(1394, TorznabCatType.Audio, "      Foreign Rap, Hip-Hop (Video)");
            AddCategoryMapping(1395, TorznabCatType.Audio, "      Foreign R'n'B (Video)");
            AddCategoryMapping(1396, TorznabCatType.Audio, "      Rap, Hip-Hop, R'n'B (DVD Video)");
            AddCategoryMapping(1397, TorznabCatType.Audio, "      Rap, Hip-Hop, R'n'B (HD Video)");
            AddCategoryMapping(65, TorznabCatType.Audio, "   Pop music");
            AddCategoryMapping(196, TorznabCatType.Audio, "      International Pop Music");
            AddCategoryMapping(689, TorznabCatType.Audio, "         Foreign pop music (lossy)");
            AddCategoryMapping(712, TorznabCatType.Audio, "         Foreign pop music (lossless)");
            AddCategoryMapping(1429, TorznabCatType.Audio, "         Foreign pop music (collections) (lossy)");
            AddCategoryMapping(711, TorznabCatType.Audio, "         East Asian pop music (lossy)");
            AddCategoryMapping(1432, TorznabCatType.Audio, "         East Asian pop music (lossless)");
            AddCategoryMapping(879, TorznabCatType.Audio, "      Domestic Pop");
            AddCategoryMapping(197, TorznabCatType.Audio, "         Patriotic Pop (lossy)");
            AddCategoryMapping(1425, TorznabCatType.Audio, "         Patriotic Pop (lossless)");
            AddCategoryMapping(1426, TorznabCatType.Audio, "         Patriotic Pop music (collections) (lossy)");
            AddCategoryMapping(1427, TorznabCatType.Audio, "         Soviet stage, Retro (lossy)");
            AddCategoryMapping(1428, TorznabCatType.Audio, "         Soviet stage, Retro (lossless)");
            AddCategoryMapping(198, TorznabCatType.Audio, "      Eurodance, Disco, Hi-NRG");
            AddCategoryMapping(703, TorznabCatType.Audio, "         Eurodance, Euro-House, Technopop (lossy)");
            AddCategoryMapping(877, TorznabCatType.Audio, "         Eurodance, Euro-House, Technopop (lossless)");
            AddCategoryMapping(199, TorznabCatType.Audio, "         Eurodance, Euro-House, Technopop (collections) (lossy)");
            AddCategoryMapping(704, TorznabCatType.Audio, "         Disco, Italo-Disco, Euro-Disco, Hi-NRG (lossy)");
            AddCategoryMapping(878, TorznabCatType.Audio, "         Disco, Italo-Disco, Euro-Disco, Hi-NRG (lossless)");
            AddCategoryMapping(1433, TorznabCatType.Audio, "         Disco, Italo-Disco, Euro-Disco, Hi-NRG (collections) (lossy)");
            AddCategoryMapping(1434, TorznabCatType.Audio, "      Video, DVD Video, HD Video (Pop)");
            AddCategoryMapping(1435, TorznabCatType.Audio, "         Patriotic Pop (Video)");
            AddCategoryMapping(1436, TorznabCatType.Audio, "         Patriotic Pop (DVD Video)");
            AddCategoryMapping(1437, TorznabCatType.Audio, "         Soviet stage, Retro (video)");
            AddCategoryMapping(1438, TorznabCatType.Audio, "         Soviet stage, Retro (DVD Video)");
            AddCategoryMapping(1439, TorznabCatType.Audio, "         Foreign pop music (Video)");
            AddCategoryMapping(1440, TorznabCatType.Audio, "         Foreign pop music (DVD Video)");
            AddCategoryMapping(1441, TorznabCatType.Audio, "         Eurodance, Disco (video)");
            AddCategoryMapping(1442, TorznabCatType.Audio, "         Eurodance, Disco (DVD Video)");
            AddCategoryMapping(1443, TorznabCatType.Audio, "         East Asian pop music (Video)");
            AddCategoryMapping(1444, TorznabCatType.Audio, "         East Asian pop music (DVD Video)");
            AddCategoryMapping(1447, TorznabCatType.Audio, "         Patriotic Pop (National concerts, Doc. Video) (Video and DVD)");
            AddCategoryMapping(1448, TorznabCatType.Audio, "         Foreign pop music (National concerts, Doc. Video) (Video and DVD)");
            AddCategoryMapping(1449, TorznabCatType.Audio, "         International Pop Music, Chanson, Eurodance, Disco (HD Video)");
            AddCategoryMapping(1450, TorznabCatType.Audio, "         Patriotic Pop Music, Chanson, Eurodance, Disco (HD Video)");
            AddCategoryMapping(1451, TorznabCatType.Audio, "      The multi-channel music and own digitization (pop music)");
            AddCategoryMapping(1452, TorznabCatType.Audio, "         Foreign pop music (own digitization)");
            AddCategoryMapping(1453, TorznabCatType.Audio, "         Eastern pop music (own digitization)");
            AddCategoryMapping(1454, TorznabCatType.Audio, "         Patriotic Pop (own digitization)");
            AddCategoryMapping(1455, TorznabCatType.Audio, "         Instrumental Pop (own digitization)");
            AddCategoryMapping(1456, TorznabCatType.Audio, "         Multichannel music (pop music)");
            AddCategoryMapping(1457, TorznabCatType.Audio, "         Foreign pop music (Hi-Res stereo)");
            AddCategoryMapping(66, TorznabCatType.Audio, "   Soundtracks, Karaoke and Minus");
            AddCategoryMapping(917, TorznabCatType.Audio, "      The arrangements of music from the game (lossy and lossless)");
            AddCategoryMapping(1400, TorznabCatType.Audio, "      Minus (lossy and lossless)");
            AddCategoryMapping(1824, TorznabCatType.Audio, "      Soundtracks");
            AddCategoryMapping(232, TorznabCatType.Audio, "         Soundtracks to movies and cartoons (mp3)");
            AddCategoryMapping(234, TorznabCatType.Audio, "         Soundtracks for games (lossy)");
            AddCategoryMapping(693, TorznabCatType.Audio, "         Soundtracks to foreign films (lossy)");
            AddCategoryMapping(880, TorznabCatType.Audio, "         Soundtracks to foreign films (lossless)");
            AddCategoryMapping(915, TorznabCatType.Audio, "         Soundtracks for games (lossless)");
            AddCategoryMapping(916, TorznabCatType.Audio, "         Informal soundtracks for games (lossy)");
            AddCategoryMapping(918, TorznabCatType.Audio, "         Informal soundtracks for films and TV series (lossy)");
            AddCategoryMapping(919, TorznabCatType.Audio, "         Soundtracks for domestic films (lossy)");
            AddCategoryMapping(920, TorznabCatType.Audio, "         Soundtracks for domestic films (lossless)");
            AddCategoryMapping(921, TorznabCatType.Audio, "         Soundtracks (own digitization)");
            AddCategoryMapping(1825, TorznabCatType.Audio, "      Karaoke");
            AddCategoryMapping(1398, TorznabCatType.Audio, "         Karaoke (Audio)");
            AddCategoryMapping(1399, TorznabCatType.Audio, "         Karaoke (Video)");
            AddCategoryMapping(67, TorznabCatType.Audio, "   Music of other genres");
            AddCategoryMapping(694, TorznabCatType.Audio, "      Proper sampling (music of other genres)");
            AddCategoryMapping(695, TorznabCatType.Audio, "      Patriotic music of other genres (lossy)");
            AddCategoryMapping(696, TorznabCatType.Audio, "      Patriotic music of other genres (lossless)");
            AddCategoryMapping(697, TorznabCatType.Audio, "      Foreign music of other genres (lossy)");
            AddCategoryMapping(1408, TorznabCatType.Audio, "      Foreign music of other genres (lossless)");
            AddCategoryMapping(1409, TorznabCatType.Audio, "      Music for ballroom dancing (lossy and lossless)");
            AddCategoryMapping(1410, TorznabCatType.Audio, "      Orthodox chants (lossy)");
            AddCategoryMapping(1411, TorznabCatType.Audio, "      Orthodox chants (lossless)");
            AddCategoryMapping(1412, TorznabCatType.Audio, "      A collection of songs for children (lossy and lossless)");
            AddCategoryMapping(1686, TorznabCatType.Audio, "         Classics for mothers and babies");
            AddCategoryMapping(1413, TorznabCatType.Audio, "      Video (Music from other genres)");
            AddCategoryMapping(1414, TorznabCatType.Audio, "      DVD Video (Music from other genres)");
            AddCategoryMapping(1415, TorznabCatType.Audio, "      Musical (lossy and lossless)");
            AddCategoryMapping(1416, TorznabCatType.Audio, "      Musical (Video and DVD Video)");
            AddCategoryMapping(1417, TorznabCatType.Audio, "      Informal and vnezhanrovye collections (lossy)");
            AddCategoryMapping(172, TorznabCatType.Audio, "   Classical and contemporary academic music");
            AddCategoryMapping(705, TorznabCatType.Audio, "      Vocal music (lossless)");
            AddCategoryMapping(706, TorznabCatType.Audio, "      Orchestral music (lossless)");
            AddCategoryMapping(707, TorznabCatType.Audio, "      Concerto for instrument and orchestra (lossless)");
            AddCategoryMapping(708, TorznabCatType.Audio, "      Chamber instrumental music (lossless)");
            AddCategoryMapping(709, TorznabCatType.Audio, "      Solo instrumental music (lossless)");
            AddCategoryMapping(710, TorznabCatType.Audio, "      Vocal and choral music (lossy)");
            AddCategoryMapping(1001, TorznabCatType.Audio, "      Proper digitization (Classical Music)");
            AddCategoryMapping(1002, TorznabCatType.Audio, "      Multichannel music (Classic and classic with a modern twist)");
            AddCategoryMapping(1003, TorznabCatType.Audio, "      Classic and classic in modern processing (Hi-Res stereo)");
            AddCategoryMapping(1008, TorznabCatType.Audio, "      Classical Music (Video)");
            AddCategoryMapping(1009, TorznabCatType.Audio, "      Classical Music (DVD and HD Video)");
            AddCategoryMapping(1362, TorznabCatType.Audio, "      Opera (Video)");
            AddCategoryMapping(1363, TorznabCatType.Audio, "      Opera (DVD and HD Video)");
            AddCategoryMapping(1364, TorznabCatType.Audio, "      Ballet and modern choreography (Video, DVD and HD Video)");
            AddCategoryMapping(1365, TorznabCatType.Audio, "      The complete collection of works and multi-disc edition (lossless)");
            AddCategoryMapping(1366, TorznabCatType.Audio, "      Opera (lossless)");
            AddCategoryMapping(1367, TorznabCatType.Audio, "      Choral music (lossless)");
            AddCategoryMapping(1368, TorznabCatType.Audio, "      The complete collection of works and multi-disc edition (lossy)");
            AddCategoryMapping(1369, TorznabCatType.Audio, "      Orchestral music (lossy)");
            AddCategoryMapping(1370, TorznabCatType.Audio, "      Chamber and solo instrumental music (lossy)");
            AddCategoryMapping(1371, TorznabCatType.Audio, "      Classics in modern processing, Classical Crossover (lossy and lossless)");
            AddCategoryMapping(175, TorznabCatType.Audio, "   Jazz, Blues");
            AddCategoryMapping(178, TorznabCatType.Audio, "      foreign Jazz");
            AddCategoryMapping(747, TorznabCatType.Audio, "         Avant-Garde Jazz, Free Improvisation (lossless)");
            AddCategoryMapping(741, TorznabCatType.Audio, "         Bop (lossless)");
            AddCategoryMapping(842, TorznabCatType.Audio, "         Early Jazz, Swing, Gypsy (lossless)");
            AddCategoryMapping(847, TorznabCatType.Audio, "         Funk, Soul, R &amp; B (lossless)");
            AddCategoryMapping(843, TorznabCatType.Audio, "         Jazz Fusion (lossless)");
            AddCategoryMapping(599, TorznabCatType.Audio, "         Mainstream Jazz, Cool (lossless)");
            AddCategoryMapping(845, TorznabCatType.Audio, "         Modern Creative, Third Stream (lossless)");
            AddCategoryMapping(846, TorznabCatType.Audio, "         Smooth, Jazz-Pop (lossless)");
            AddCategoryMapping(620, TorznabCatType.Audio, "         Vocal Jazz (lossless)");
            AddCategoryMapping(844, TorznabCatType.Audio, "         World Fusion, Ethnic Jazz (lossless)");
            AddCategoryMapping(673, TorznabCatType.Audio, "         Foreign jazz (lossy)");
            AddCategoryMapping(848, TorznabCatType.Audio, "         Collections of foreign jazz (lossless)");
            AddCategoryMapping(690, TorznabCatType.Audio, "      foreign blues");
            AddCategoryMapping(839, TorznabCatType.Audio, "         Blues-rock (lossless)");
            AddCategoryMapping(713, TorznabCatType.Audio, "         Blues (Texas, Chicago, Modern and Others) (lossless)");
            AddCategoryMapping(840, TorznabCatType.Audio, "         Roots, Pre-War Blues, Early R &amp; B, Gospel (lossless)");
            AddCategoryMapping(691, TorznabCatType.Audio, "         Foreign blues (lossy)");
            AddCategoryMapping(841, TorznabCatType.Audio, "         Foreign blues (collections; Tribute VA) (lossless)");
            AddCategoryMapping(849, TorznabCatType.Audio, "      Domestic jazz and blues");
            AddCategoryMapping(850, TorznabCatType.Audio, "         Domestic Jazz (lossless)");
            AddCategoryMapping(851, TorznabCatType.Audio, "         Domestic jazz (lossy)");
            AddCategoryMapping(853, TorznabCatType.Audio, "         Domestic Blues (lossless)");
            AddCategoryMapping(854, TorznabCatType.Audio, "         Domestic Blues (lossy)");
            AddCategoryMapping(1013, TorznabCatType.Audio, "      The multi-channel music and own digitization (Jazz and Blues)");
            AddCategoryMapping(1014, TorznabCatType.Audio, "         Multichannel music (Jazz and Blues)");
            AddCategoryMapping(1015, TorznabCatType.Audio, "         Foreign Jazz and Blues (Hi-Res stereo)");
            AddCategoryMapping(1016, TorznabCatType.Audio, "         Proper digitization (Jazz and Blues)");
            AddCategoryMapping(1458, TorznabCatType.Audio, "      Video, DVD Video, HD Video (Jazz and Blues)");
            AddCategoryMapping(1459, TorznabCatType.Audio, "         Jazz and Blues (Video)");
            AddCategoryMapping(1460, TorznabCatType.Audio, "         Jazz and Blues (DVD Video)");
            AddCategoryMapping(1461, TorznabCatType.Audio, "         Jazz and Blues (HD Video)");
            AddCategoryMapping(204, TorznabCatType.Audio, "   New Age, Relax, Meditative &amp; Flamenco");
            AddCategoryMapping(207, TorznabCatType.Audio, "      NewAge &amp; Meditative (lossy)");
            AddCategoryMapping(1004, TorznabCatType.Audio, "      NewAge &amp; Meditative (lossless)");
            AddCategoryMapping(1005, TorznabCatType.Audio, "      Flamenco and acoustic guitar (lossy)");
            AddCategoryMapping(1006, TorznabCatType.Audio, "      Flamenco and acoustic guitar (lossless)");
            AddCategoryMapping(1385, TorznabCatType.Audio, "      New Age, Relax, Meditative &amp; Flamenco (Video)");
            AddCategoryMapping(1386, TorznabCatType.Audio, "      New Age, Relax, Meditative &amp; Flamenco (DVD and HD Video)");
            AddCategoryMapping(1007, TorznabCatType.Audio, "      Sounds of nature");
            AddCategoryMapping(701, TorznabCatType.Audio, "   Chanson, Author and military songs");
            AddCategoryMapping(702, TorznabCatType.Audio, "      Domestic chanson (lossy)");
            AddCategoryMapping(891, TorznabCatType.Audio, "      Domestic chanson (lossless)");
            AddCategoryMapping(892, TorznabCatType.Audio, "      Military song (lossy)");
            AddCategoryMapping(893, TorznabCatType.Audio, "      Chanson (lossy)");
            AddCategoryMapping(1401, TorznabCatType.Audio, "      Collections domestic chanson (lossy)");
            AddCategoryMapping(1402, TorznabCatType.Audio, "      Military song (lossless)");
            AddCategoryMapping(1403, TorznabCatType.Audio, "      Chanson (lossless)");
            AddCategoryMapping(1404, TorznabCatType.Audio, "      Minstrels and roleviki (lossy and lossless)");
            AddCategoryMapping(1405, TorznabCatType.Audio, "      Proper digitization (Chanson, and Bards) lossless");
            AddCategoryMapping(1406, TorznabCatType.Audio, "      Videos (Chanson, and Bards)");
            AddCategoryMapping(1407, TorznabCatType.Audio, "      DVD Video (Chanson, and Bards)");
            AddCategoryMapping(1430, TorznabCatType.Audio, "      Foreign chanson (lossy)");
            AddCategoryMapping(1431, TorznabCatType.Audio, "      Foreign chanson (lossless)");
            AddCategoryMapping(1445, TorznabCatType.Audio, "      Foreign chanson (Video)");
            AddCategoryMapping(1446, TorznabCatType.Audio, "      Foreign chanson (DVD Video)");
            AddCategoryMapping(865, TorznabCatType.Audio, "   Rock music");
            AddCategoryMapping(62, TorznabCatType.Audio, "      foreign Rock");
            AddCategoryMapping(859, TorznabCatType.Audio, "         AOR (Melodic Hard Rock, Arena rock) (lossless)");
            AddCategoryMapping(860, TorznabCatType.Audio, "         AOR (Melodic Hard Rock, Arena rock) (lossy)");
            AddCategoryMapping(855, TorznabCatType.Audio, "         Classic Rock &amp; Hard Rock (lossless)");
            AddCategoryMapping(856, TorznabCatType.Audio, "         Classic Rock &amp; Hard Rock (lossy)");
            AddCategoryMapping(864, TorznabCatType.Audio, "         Instrumental Guitar Rock (lossy)");
            AddCategoryMapping(1027, TorznabCatType.Audio, "         Instrumental Guitar Rock (lossless)");
            AddCategoryMapping(858, TorznabCatType.Audio, "         Folk-Rock (lossless)");
            AddCategoryMapping(675, TorznabCatType.Audio, "         Folk-Rock (lossy)");
            AddCategoryMapping(861, TorznabCatType.Audio, "         Pop-Rock &amp; Soft Rock (lossless)");
            AddCategoryMapping(676, TorznabCatType.Audio, "         Pop-Rock &amp; Soft Rock (lossy)");
            AddCategoryMapping(857, TorznabCatType.Audio, "         Progressive &amp; Art-Rock (lossless)");
            AddCategoryMapping(674, TorznabCatType.Audio, "         Progressive &amp; Art-Rock (lossy)");
            AddCategoryMapping(863, TorznabCatType.Audio, "         Rockabilly, Psychobilly, Rock'n'Roll (lossless)");
            AddCategoryMapping(730, TorznabCatType.Audio, "         Rockabilly, Psychobilly, Rock'n'Roll (lossy)");
            AddCategoryMapping(1028, TorznabCatType.Audio, "         East Asian Rock (lossless)");
            AddCategoryMapping(862, TorznabCatType.Audio, "         East Asian rock (lossy)");
            AddCategoryMapping(1858, TorznabCatType.Audio, "         Collections of foreign rock (lossless)");
            AddCategoryMapping(1859, TorznabCatType.Audio, "         Collections of foreign rock (lossy)");
            AddCategoryMapping(187, TorznabCatType.Audio, "      Domestic Rock");
            AddCategoryMapping(872, TorznabCatType.Audio, "         Rock, Punk, Alternative (lossless)");
            AddCategoryMapping(190, TorznabCatType.Audio, "         Rock, Punk, Alternative (lossy)");
            AddCategoryMapping(1489, TorznabCatType.Audio, "         Metal (lossless)");
            AddCategoryMapping(191, TorznabCatType.Audio, "         Metal (lossy)");
            AddCategoryMapping(1490, TorznabCatType.Audio, "         Rock on the languages ??of the peoples xUSSR (lossless)");
            AddCategoryMapping(1491, TorznabCatType.Audio, "         Rock on the languages ??of the peoples xUSSR (lossy)");
            AddCategoryMapping(866, TorznabCatType.Audio, "      foreign Metal");
            AddCategoryMapping(727, TorznabCatType.Audio, "         Black (lossy)");
            AddCategoryMapping(728, TorznabCatType.Audio, "         Death, Doom (lossy)");
            AddCategoryMapping(729, TorznabCatType.Audio, "         Heavy, Power, Progressive (lossy)");
            AddCategoryMapping(871, TorznabCatType.Audio, "         Heavy, Power, Progressive (lossless)");
            AddCategoryMapping(1462, TorznabCatType.Audio, "         Avant-garde, Experimental Metal (lossless)");
            AddCategoryMapping(1463, TorznabCatType.Audio, "         Avant-garde, Experimental Metal (lossy)");
            AddCategoryMapping(1464, TorznabCatType.Audio, "         Black (lossless)");
            AddCategoryMapping(1466, TorznabCatType.Audio, "         Death, Doom (lossless)");
            AddCategoryMapping(1468, TorznabCatType.Audio, "         Folk, Pagan, Viking (lossless)");
            AddCategoryMapping(1469, TorznabCatType.Audio, "         Folk, Pagan, Viking (lossy)");
            AddCategoryMapping(1470, TorznabCatType.Audio, "         Gothic Metal (lossless)");
            AddCategoryMapping(1471, TorznabCatType.Audio, "         Gothic Metal (lossy)");
            AddCategoryMapping(1472, TorznabCatType.Audio, "         Grind, Brutal Death (lossless)");
            AddCategoryMapping(1473, TorznabCatType.Audio, "         Grind, Brutal Death (lossy)");
            AddCategoryMapping(1474, TorznabCatType.Audio, "         Sludge, Stoner, Post-Metal (lossless)");
            AddCategoryMapping(1475, TorznabCatType.Audio, "         Sludge, Stoner, Post-Metal (lossy)");
            AddCategoryMapping(1476, TorznabCatType.Audio, "         Thrash, Speed ??(lossless)");
            AddCategoryMapping(1477, TorznabCatType.Audio, "         Thrash, Speed ??(lossy)");
            AddCategoryMapping(1478, TorznabCatType.Audio, "         Collections \"Foreign Metal\" (lossless)");
            AddCategoryMapping(1479, TorznabCatType.Audio, "         Collections \"Foreign Metal\" (lossy)");
            AddCategoryMapping(867, TorznabCatType.Audio, "      Foreign Alternative, Punk, Independent");
            AddCategoryMapping(185, TorznabCatType.Audio, "         Alternative &amp; Nu-metal (lossless)");
            AddCategoryMapping(868, TorznabCatType.Audio, "         Alternative &amp; Nu-metal (lossy)");
            AddCategoryMapping(869, TorznabCatType.Audio, "         Indie, Post-Rock &amp; Post-Punk (lossless)");
            AddCategoryMapping(870, TorznabCatType.Audio, "         Indie, Post-Rock &amp; Post-Punk (lossy)");
            AddCategoryMapping(873, TorznabCatType.Audio, "         Avant-garde, Experimental Rock (lossy)");
            AddCategoryMapping(874, TorznabCatType.Audio, "         Punk (lossy)");
            AddCategoryMapping(875, TorznabCatType.Audio, "         Punk (lossless)");
            AddCategoryMapping(876, TorznabCatType.Audio, "         Avant-garde, Experimental Rock (lossless)");
            AddCategoryMapping(1480, TorznabCatType.Audio, "         Hardcore (lossless)");
            AddCategoryMapping(1481, TorznabCatType.Audio, "         Hardcore (lossy)");
            AddCategoryMapping(1482, TorznabCatType.Audio, "         Industrial &amp; Post-industrial (lossless)");
            AddCategoryMapping(1483, TorznabCatType.Audio, "         Industrial &amp; Post-industrial (lossy)");
            AddCategoryMapping(1484, TorznabCatType.Audio, "         Emocore, Post-hardcore, Metalcore (lossless)");
            AddCategoryMapping(1485, TorznabCatType.Audio, "         Emocore, Post-hardcore, Metalcore (lossy)");
            AddCategoryMapping(1486, TorznabCatType.Audio, "         Gothic Rock &amp; Dark Folk (lossless)");
            AddCategoryMapping(1487, TorznabCatType.Audio, "         Gothic Rock &amp; Dark Folk (lossy)");
            AddCategoryMapping(1492, TorznabCatType.Audio, "      The multi-channel music and own digitization (Rock)");
            AddCategoryMapping(1493, TorznabCatType.Audio, "         Foreign rock (own digitization)");
            AddCategoryMapping(1494, TorznabCatType.Audio, "         Domestic Rock (own digitization)");
            AddCategoryMapping(1495, TorznabCatType.Audio, "         Foreign and domestic rock (multichannel music)");
            AddCategoryMapping(1496, TorznabCatType.Audio, "         Foreign rock (Hi-Res stereo)");
            AddCategoryMapping(1497, TorznabCatType.Audio, "         Conversion Quadraphonic (multichannel music)");
            AddCategoryMapping(1498, TorznabCatType.Audio, "         Conversion SACD (multi-channel music)");
            AddCategoryMapping(1499, TorznabCatType.Audio, "         Conversions in Blu-Ray (multichannel music)");
            AddCategoryMapping(1500, TorznabCatType.Audio, "         Apmiksy-Upmixes / downmix-Downmix (multi-channel and Hi-Res stereo music)");
            AddCategoryMapping(1501, TorznabCatType.Audio, "      Video, DVD Video, HD Video (Rock)");
            AddCategoryMapping(1502, TorznabCatType.Audio, "         Rock (Video)");
            AddCategoryMapping(1503, TorznabCatType.Audio, "         Rock (DVD Video)");
            AddCategoryMapping(1504, TorznabCatType.Audio, "         Rock (Unofficial DVD Video)");
            AddCategoryMapping(1505, TorznabCatType.Audio, "         Metal (Video)");
            AddCategoryMapping(1506, TorznabCatType.Audio, "         Metal (DVD Video)");
            AddCategoryMapping(1507, TorznabCatType.Audio, "         Metal (Unofficial DVD Video)");
            AddCategoryMapping(1508, TorznabCatType.Audio, "         Alternative, Punk, Independent (Video)");
            AddCategoryMapping(1509, TorznabCatType.Audio, "         Alternative, Punk, Independent (DVD Video)");
            AddCategoryMapping(1510, TorznabCatType.Audio, "         Alternative, Punk, Independent (Unofficial DVD Video)");
            AddCategoryMapping(1511, TorznabCatType.Audio, "         Domestic Rock, Punk, Alternative (Video)");
            AddCategoryMapping(1512, TorznabCatType.Audio, "         Domestic Rock, Punk, Alternative (DVD Video)");
            AddCategoryMapping(1513, TorznabCatType.Audio, "         Domestic Metal (Video)");
            AddCategoryMapping(1514, TorznabCatType.Audio, "         Domestic Metal (DVD Video)");
            AddCategoryMapping(1515, TorznabCatType.Audio, "         Domestic Rock, Punk, Alternative, Metal (Unofficial DVD Video)");
            AddCategoryMapping(1516, TorznabCatType.Audio, "         Rock (HD Video)");
            AddCategoryMapping(905, TorznabCatType.Audio, "   Folklore, Folk and World Music");
            AddCategoryMapping(906, TorznabCatType.Audio, "      Eastern European Folk (lossy)");
            AddCategoryMapping(907, TorznabCatType.Audio, "      Eastern European Folk (lossless)");
            AddCategoryMapping(908, TorznabCatType.Audio, "      Western European folk (lossy)");
            AddCategoryMapping(909, TorznabCatType.Audio, "      Western European folk (lossless)");
            AddCategoryMapping(910, TorznabCatType.Audio, "      Klezmer and Jewish folklore (lossy and lossless)");
            AddCategoryMapping(911, TorznabCatType.Audio, "      Country, Bluegrass (lossy)");
            AddCategoryMapping(912, TorznabCatType.Audio, "      Country, Bluegrass (lossless)");
            AddCategoryMapping(1372, TorznabCatType.Audio, "      World Music Siberia, Central Asia and East Asia (lossy)");
            AddCategoryMapping(1373, TorznabCatType.Audio, "      World Music Siberia, Central Asia and East Asia (lossless)");
            AddCategoryMapping(1374, TorznabCatType.Audio, "      World Music India (lossy)");
            AddCategoryMapping(1375, TorznabCatType.Audio, "      World Music India (lossless)");
            AddCategoryMapping(1376, TorznabCatType.Audio, "      World Music Africa and the Middle East (lossy)");
            AddCategoryMapping(1377, TorznabCatType.Audio, "      World Music Africa and the Middle East (lossless)");
            AddCategoryMapping(1378, TorznabCatType.Audio, "      Ethnic Caucasus and Transcaucasia music (lossy and lossless)");
            AddCategoryMapping(1379, TorznabCatType.Audio, "      World Music Americas (lossy)");
            AddCategoryMapping(1380, TorznabCatType.Audio, "      World Music Americas (lossless)");
            AddCategoryMapping(1381, TorznabCatType.Audio, "      World Music Australia, the Pacific and Indian Oceans (lossy and lossless)");
            AddCategoryMapping(1382, TorznabCatType.Audio, "      Folklore, Folk and World Music (Video)");
            AddCategoryMapping(1383, TorznabCatType.Audio, "      Folklore, Folk and World Music (DVD Video)");
            AddCategoryMapping(1384, TorznabCatType.Audio, "      Folklore, Folk and World Music (HD Video)");
            AddCategoryMapping(1857, TorznabCatType.Audio, "      Folklore, Folk and World Music (own digitization)");
            AddCategoryMapping(985, TorznabCatType.Audio, "   Reggae, Ska, Dub");
            AddCategoryMapping(986, TorznabCatType.Audio, "      Rocksteady, Early Reggae, Ska-Jazz, Trad.Ska (lossy and lossless)");
            AddCategoryMapping(987, TorznabCatType.Audio, "      Punky-Reggae, Rocksteady-Punk, Ska Revival (lossy)");
            AddCategoryMapping(988, TorznabCatType.Audio, "      3rd Wave Ska (lossy)");
            AddCategoryMapping(989, TorznabCatType.Audio, "      Ska-Punk, Ska-Core (lossy)");
            AddCategoryMapping(990, TorznabCatType.Audio, "      Reggae (lossy)");
            AddCategoryMapping(1026, TorznabCatType.Audio, "      Dub (lossy)");
            AddCategoryMapping(991, TorznabCatType.Audio, "      Dancehall, Raggamuffin (lossy)");
            AddCategoryMapping(992, TorznabCatType.Audio, "      Reggae, Dancehall, Dub (lossless)");
            AddCategoryMapping(993, TorznabCatType.Audio, "      Ska, Ska-Punk, Ska-Jazz (lossless)");
            AddCategoryMapping(994, TorznabCatType.Audio, "      Domestic reggae, dub (lossy and lossless)");
            AddCategoryMapping(995, TorznabCatType.Audio, "      Domestic ska music (lossy and lossless)");
            AddCategoryMapping(997, TorznabCatType.Audio, "      Reggae, Ska, Dub (own digitization)");
            AddCategoryMapping(998, TorznabCatType.Audio, "      Reggae, Ska, Dub (compilation) (lossy)");
            AddCategoryMapping(999, TorznabCatType.Audio, "      Reggae, Ska, Dub (Video)");
            AddCategoryMapping(1000, TorznabCatType.Audio, "      Reggae, Ska, Dub (DVD and HD Video)");
            AddCategoryMapping(1418, TorznabCatType.Audio, "   Sheet Music literature");
            AddCategoryMapping(1419, TorznabCatType.Audio, "      Academic Music (Notes and Media CD)");
            AddCategoryMapping(1420, TorznabCatType.Audio, "      More destinations (notes, tablature)");
            AddCategoryMapping(1421, TorznabCatType.Audio, "      Self and School");
            AddCategoryMapping(1422, TorznabCatType.Audio, "      Songbooks (Songbooks)");
            AddCategoryMapping(1423, TorznabCatType.Audio, "      Music Literature and Theory");
            AddCategoryMapping(1424, TorznabCatType.Audio, "      music magazines");
            AddCategoryMapping(1602, TorznabCatType.Audio, "   Audio for Apple devices");
            AddCategoryMapping(1603, TorznabCatType.Audio, "      Audiobooks (AAC, ALAC)");
            AddCategoryMapping(1604, TorznabCatType.Audio, "      Music Lossless (ALAC)");
            AddCategoryMapping(1605, TorznabCatType.Audio, "      Music Lossy (AAC)");
            AddCategoryMapping(1606, TorznabCatType.Audio, "      Music Lossy (AAC) (Singles, EPs)");
            AddCategoryMapping(113, TorznabCatType.Books, "Books, Audio Books, Journals");
            AddCategoryMapping(561, TorznabCatType.Books, "   Books, Audio Books, Journals (General Discussion)");
            AddCategoryMapping(316, TorznabCatType.Books, "      Archive (Books, Audio Books, Journals)");
            AddCategoryMapping(1620, TorznabCatType.Books, "      Meditation");
            AddCategoryMapping(1802, TorznabCatType.Books, "      Historiography");
            AddCategoryMapping(116, TorznabCatType.AudioAudiobook, "   Audiobooks");
            AddCategoryMapping(1231, TorznabCatType.AudioAudiobook, "      Audio, history, memoirs (Audiobooks)");
            AddCategoryMapping(119, TorznabCatType.AudioAudiobook, "         Audio and literary readings (Audiobooks)");
            AddCategoryMapping(1232, TorznabCatType.AudioAudiobook, "         Lots of great people (Audiobooks)");
            AddCategoryMapping(1233, TorznabCatType.AudioAudiobook, "         Historical book (Audiobooks)");
            AddCategoryMapping(321, TorznabCatType.AudioAudiobook, "      Science fiction, fantasy, mystery, horror, fanfiction (Audiobooks)");
            AddCategoryMapping(1234, TorznabCatType.AudioAudiobook, "         Russian fiction, fantasy, mystery, horror, fanfiction (audiobook)");
            AddCategoryMapping(1235, TorznabCatType.AudioAudiobook, "         Foreign fiction, fantasy, mystery, horror, fanfiction (audiobook)");
            AddCategoryMapping(317, TorznabCatType.AudioAudiobook, "      Fiction (Audiobooks)");
            AddCategoryMapping(1236, TorznabCatType.AudioAudiobook, "         Poetry (audiobook)");
            AddCategoryMapping(118, TorznabCatType.AudioAudiobook, "         Foreign literature (audiobook)");
            AddCategoryMapping(117, TorznabCatType.AudioAudiobook, "         Russian Literature (audiobook)");
            AddCategoryMapping(120, TorznabCatType.AudioAudiobook, "         Children's Literature (audiobook)");
            AddCategoryMapping(1237, TorznabCatType.AudioAudiobook, "         Detectives, Adventure, Thriller, Action (audiobook)");
            AddCategoryMapping(318, TorznabCatType.AudioAudiobook, "            Detectives (audiobook)");
            AddCategoryMapping(322, TorznabCatType.AudioAudiobook, "      Psychology. philosophy, religion (Audiobooks)");
            AddCategoryMapping(1238, TorznabCatType.AudioAudiobook, "         Religion (Audio)");
            AddCategoryMapping(1239, TorznabCatType.AudioAudiobook, "            Orthodox (Audio)");
            AddCategoryMapping(1240, TorznabCatType.AudioAudiobook, "            Islam (Audio)");
            AddCategoryMapping(1241, TorznabCatType.AudioAudiobook, "            Other traditional religion (Audio)");
            AddCategoryMapping(1242, TorznabCatType.AudioAudiobook, "            Nontraditional religious philosophies (Audio)");
            AddCategoryMapping(1243, TorznabCatType.AudioAudiobook, "      Educational, scientific and popular literature (Audio)");
            AddCategoryMapping(1244, TorznabCatType.AudioAudiobook, "      Audiobooks in lossless-format");
            AddCategoryMapping(323, TorznabCatType.AudioAudiobook, "      Business (Audio)");
            AddCategoryMapping(319, TorznabCatType.AudioAudiobook, "      Historical literature (Audiobooks)");
            AddCategoryMapping(1245, TorznabCatType.AudioAudiobook, "      Miscellaneous (Audiobooks)");
            AddCategoryMapping(1622, TorznabCatType.AudioAudiobook, "      Meditation (Audio)");
            AddCategoryMapping(1626, TorznabCatType.AudioAudiobook, "      publicism");
            AddCategoryMapping(1627, TorznabCatType.AudioAudiobook, "      Satire, humor");
            AddCategoryMapping(324, TorznabCatType.AudioAudiobook, "      Archive and nekonditsiya audiobooks");
            AddCategoryMapping(670, TorznabCatType.Books, "   Books and magazines");
            AddCategoryMapping(671, TorznabCatType.Books, "      Esoteric Tarot, Feng Shui");
            AddCategoryMapping(885, TorznabCatType.Books, "      Film, TV, animation");
            AddCategoryMapping(886, TorznabCatType.Books, "      Drawing, Graphic Design");
            AddCategoryMapping(887, TorznabCatType.Books, "      Photo and video shooting");
            AddCategoryMapping(888, TorznabCatType.Books, "      Astrology");
            AddCategoryMapping(1865, TorznabCatType.Books, "      Fashion. Style. Etiquette");
            AddCategoryMapping(889, TorznabCatType.Books, "      Celebrities and idols");
            AddCategoryMapping(890, TorznabCatType.Books, "      Miscellaneous");
            AddCategoryMapping(982, TorznabCatType.Books, "      Magazines and newspapers (general section)");
            AddCategoryMapping(566, TorznabCatType.Books, "         Men's magazines");
            AddCategoryMapping(1204, TorznabCatType.Books, "         For women (magazines and books)");
            AddCategoryMapping(1793, TorznabCatType.Books, "         Popular science magazines");
            AddCategoryMapping(1794, TorznabCatType.Books, "         Journals of Electrical and Electronics");
            AddCategoryMapping(1795, TorznabCatType.Books, "         Housekeeping (logs)");
            AddCategoryMapping(1796, TorznabCatType.Books, "         Hobbies (logs)");
            AddCategoryMapping(1797, TorznabCatType.Books, "         Other magazines");
            AddCategoryMapping(1205, TorznabCatType.Books, "      Travel and tourism");
            AddCategoryMapping(258, TorznabCatType.Books, "      For children, parents and teachers");
            AddCategoryMapping(1685, TorznabCatType.Books, "         Tutorials (General)");
            AddCategoryMapping(681, TorznabCatType.Books, "         Textbooks for kindergarten and elementary school (up to class 4)");
            AddCategoryMapping(682, TorznabCatType.Books, "         Textbooks for high school (grades 5-11)");
            AddCategoryMapping(683, TorznabCatType.Books, "         Teachers and educators");
            AddCategoryMapping(684, TorznabCatType.Books, "         Popular science and cognitive literature (for children)");
            AddCategoryMapping(685, TorznabCatType.Books, "         Leisure and creativity");
            AddCategoryMapping(686, TorznabCatType.Books, "         Education and development");
            AddCategoryMapping(687, TorznabCatType.Books, "         Hood. lit-ra for preschool and elementary grades");
            AddCategoryMapping(688, TorznabCatType.Books, "         Hood. lit-ra for the middle and upper classes");
            AddCategoryMapping(731, TorznabCatType.Books, "      Sports, physical training, martial arts");
            AddCategoryMapping(738, TorznabCatType.Books, "         Autosport. Motorcycling. Cycling (literature)");
            AddCategoryMapping(737, TorznabCatType.Books, "         Martial arts, martial arts (literature)");
            AddCategoryMapping(734, TorznabCatType.Books, "         Team sports (literature)");
            AddCategoryMapping(1854, TorznabCatType.Books, "         Athletics. Swimming. Gymnastics. Weightlifting. Rowing (literature)");
            AddCategoryMapping(739, TorznabCatType.Books, "         Sport Editions");
            AddCategoryMapping(736, TorznabCatType.Books, "         Fitness, fitness, bodybuilding (literature)");
            AddCategoryMapping(732, TorznabCatType.Books, "         Football (literature)");
            AddCategoryMapping(733, TorznabCatType.Books, "         Hockey (literature)");
            AddCategoryMapping(735, TorznabCatType.Books, "         Chess. Checkers (literature)");
            AddCategoryMapping(1855, TorznabCatType.Books, "         Extreme sports (literature)");
            AddCategoryMapping(649, TorznabCatType.Books, "      Humanitarian sciences");
            AddCategoryMapping(650, TorznabCatType.Books, "         Arts. Cultural");
            AddCategoryMapping(651, TorznabCatType.Books, "         Folklore. Epic. Mythology");
            AddCategoryMapping(652, TorznabCatType.Books, "         literary criticism");
            AddCategoryMapping(653, TorznabCatType.Books, "         Linguistics");
            AddCategoryMapping(654, TorznabCatType.Books, "         Philosophy");
            AddCategoryMapping(655, TorznabCatType.Books, "         Political science");
            AddCategoryMapping(656, TorznabCatType.Books, "         Sociology");
            AddCategoryMapping(657, TorznabCatType.Books, "         Journalism, Journalism");
            AddCategoryMapping(658, TorznabCatType.Books, "         Business, Management");
            AddCategoryMapping(659, TorznabCatType.Books, "         Marketing");
            AddCategoryMapping(660, TorznabCatType.Books, "         Economy");
            AddCategoryMapping(661, TorznabCatType.Books, "         Finance");
            AddCategoryMapping(662, TorznabCatType.Books, "         Jurisprudence. Right. criminalistics");
            AddCategoryMapping(664, TorznabCatType.Books, "      Historical sciences");
            AddCategoryMapping(665, TorznabCatType.Books, "         Historical person");
            AddCategoryMapping(807, TorznabCatType.Books, "         Historical sources");
            AddCategoryMapping(808, TorznabCatType.Books, "         Alternative historical theories");
            AddCategoryMapping(809, TorznabCatType.Books, "         Archeology");
            AddCategoryMapping(810, TorznabCatType.Books, "         Ancient world. Antiquity");
            AddCategoryMapping(811, TorznabCatType.Books, "         Middle Ages");
            AddCategoryMapping(812, TorznabCatType.Books, "         The history of modern and contemporary");
            AddCategoryMapping(813, TorznabCatType.Books, "         History of Europe");
            AddCategoryMapping(814, TorznabCatType.Books, "         History of Asia and Africa");
            AddCategoryMapping(815, TorznabCatType.Books, "         The history of America, Australia, Oceania");
            AddCategoryMapping(816, TorznabCatType.Books, "         Russian history");
            AddCategoryMapping(817, TorznabCatType.Books, "         The era of the Soviet Union");
            AddCategoryMapping(818, TorznabCatType.Books, "         History of the former Soviet Union");
            AddCategoryMapping(819, TorznabCatType.Books, "         Ethnography, anthropology");
            AddCategoryMapping(820, TorznabCatType.Books, "         International relationships. Diplomacy");
            AddCategoryMapping(1856, TorznabCatType.Books, "         Methodology and philosophy of history");
            AddCategoryMapping(255, TorznabCatType.Books, "      Accurate, natural and engineering sciences");
            AddCategoryMapping(648, TorznabCatType.Books, "         Geography / Geology / Geodesy");
            AddCategoryMapping(672, TorznabCatType.Books, "         Physics");
            AddCategoryMapping(723, TorznabCatType.Books, "         Astronomy");
            AddCategoryMapping(725, TorznabCatType.Books, "         Aviation / Astronautics");
            AddCategoryMapping(726, TorznabCatType.Books, "         Mathematics");
            AddCategoryMapping(748, TorznabCatType.Books, "         Welding / Soldering / Non-Destructive Testing");
            AddCategoryMapping(749, TorznabCatType.Books, "         Architecture / construction / engineering services");
            AddCategoryMapping(750, TorznabCatType.Books, "         Biology / Ecology");
            AddCategoryMapping(751, TorznabCatType.Books, "         Chemistry / Biochemistry");
            AddCategoryMapping(821, TorznabCatType.Books, "         Electronics / Radio");
            AddCategoryMapping(822, TorznabCatType.Books, "         Diagrams and service manuals (original documents)");
            AddCategoryMapping(823, TorznabCatType.Books, "         engineering");
            AddCategoryMapping(824, TorznabCatType.Books, "         Automation / Robotics");
            AddCategoryMapping(825, TorznabCatType.Books, "         Metallurgy / Materials");
            AddCategoryMapping(826, TorznabCatType.Books, "         Mechanics, Strength of Materials");
            AddCategoryMapping(827, TorznabCatType.Books, "         Energy / Electrical");
            AddCategoryMapping(828, TorznabCatType.Books, "         Oil, gas and chemical industry");
            AddCategoryMapping(829, TorznabCatType.Books, "         Agriculture and food industry");
            AddCategoryMapping(836, TorznabCatType.Books, "         Railway deal");
            AddCategoryMapping(837, TorznabCatType.Books, "         Normative documents");
            AddCategoryMapping(838, TorznabCatType.Books, "         Journals: scientific, popular, radio and others.");
            AddCategoryMapping(668, TorznabCatType.Books, "      Warfare");
            AddCategoryMapping(669, TorznabCatType.Books, "         History of the Second World War");
            AddCategoryMapping(830, TorznabCatType.Books, "         Militaria");
            AddCategoryMapping(831, TorznabCatType.Books, "         Military history");
            AddCategoryMapping(832, TorznabCatType.Books, "         Military equipment");
            AddCategoryMapping(833, TorznabCatType.Books, "         Weapon");
            AddCategoryMapping(834, TorznabCatType.Books, "         Educational literature");
            AddCategoryMapping(835, TorznabCatType.Books, "         The special services of the world");
            AddCategoryMapping(666, TorznabCatType.Books, "      Faith and Religion");
            AddCategoryMapping(667, TorznabCatType.Books, "         Christianity");
            AddCategoryMapping(881, TorznabCatType.Books, "         Islam");
            AddCategoryMapping(882, TorznabCatType.Books, "         Religions of India, Tibet and East Asia / Judaism");
            AddCategoryMapping(883, TorznabCatType.Books, "         Nontraditional religious, spiritual and mystical teachings");
            AddCategoryMapping(884, TorznabCatType.Books, "         Religious Studies. History of Religions. Atheism");
            AddCategoryMapping(1206, TorznabCatType.Books, "      Psychology");
            AddCategoryMapping(1207, TorznabCatType.Books, "         General and Applied Psychology");
            AddCategoryMapping(1208, TorznabCatType.Books, "         Psychotherapy and counseling");
            AddCategoryMapping(1209, TorznabCatType.Books, "         Psychological diagnostics and therapy");
            AddCategoryMapping(1210, TorznabCatType.Books, "         Social psychology and psychology of relationships");
            AddCategoryMapping(1211, TorznabCatType.Books, "         Training and Coaching");
            AddCategoryMapping(1212, TorznabCatType.Books, "         Self-development and self-improvement");
            AddCategoryMapping(1213, TorznabCatType.Books, "         Popular psychology");
            AddCategoryMapping(1214, TorznabCatType.Books, "         Sexology. Relations between the sexes");
            AddCategoryMapping(254, TorznabCatType.Books, "      Collecting, hobby and hobbies");
            AddCategoryMapping(633, TorznabCatType.Books, "         Collecting and auxiliary ist. discipline");
            AddCategoryMapping(634, TorznabCatType.Books, "         Embroidery");
            AddCategoryMapping(635, TorznabCatType.Books, "         Knitting");
            AddCategoryMapping(636, TorznabCatType.Books, "         Sewing, patchwork");
            AddCategoryMapping(637, TorznabCatType.Books, "         lace");
            AddCategoryMapping(638, TorznabCatType.Books, "         Beading");
            AddCategoryMapping(639, TorznabCatType.Books, "         Paper art");
            AddCategoryMapping(640, TorznabCatType.Books, "         Other arts and crafts");
            AddCategoryMapping(641, TorznabCatType.Books, "         Pets and aquariums");
            AddCategoryMapping(642, TorznabCatType.Books, "         Hunting and fishing");
            AddCategoryMapping(598, TorznabCatType.Books, "         Cooking (Book)");
            AddCategoryMapping(312, TorznabCatType.Books, "         Cooking (newspapers and magazines)");
            AddCategoryMapping(643, TorznabCatType.Books, "         Modelling");
            AddCategoryMapping(644, TorznabCatType.Books, "         Farmland / Floriculture");
            AddCategoryMapping(645, TorznabCatType.Books, "         Repair, private construction, design of interiors");
            AddCategoryMapping(1866, TorznabCatType.Books, "         Woodworking");
            AddCategoryMapping(646, TorznabCatType.Books, "         Board games");
            AddCategoryMapping(647, TorznabCatType.Books, "         Other hobbies");
            AddCategoryMapping(1786, TorznabCatType.Books, "      Nonfiction");
            AddCategoryMapping(623, TorznabCatType.Books, "      Fiction");
            AddCategoryMapping(624, TorznabCatType.Books, "         Russian Literature (books)");
            AddCategoryMapping(627, TorznabCatType.Books, "         The detective, thriller (book)");
            AddCategoryMapping(1624, TorznabCatType.Books, "            Militants (Books)");
            AddCategoryMapping(1625, TorznabCatType.Books, "            Detectives (Books)");
            AddCategoryMapping(628, TorznabCatType.Books, "         Female Novel (Book)");
            AddCategoryMapping(631, TorznabCatType.Books, "         Adventure (book)");
            AddCategoryMapping(632, TorznabCatType.Books, "         Literary magazines");
            AddCategoryMapping(1611, TorznabCatType.Books, "         Fiction / Fantasy / Mystic (book)");
            AddCategoryMapping(629, TorznabCatType.Books, "            Domestic science fiction / fantasy / mystic");
            AddCategoryMapping(630, TorznabCatType.Books, "            International science fiction / fantasy / mystic");
            AddCategoryMapping(1612, TorznabCatType.Books, "         Foreign literature (books)");
            AddCategoryMapping(625, TorznabCatType.Books, "            Foreign literature (up to 1900)");
            AddCategoryMapping(626, TorznabCatType.Books, "            Foreign literature (XX and XXI century)");
            AddCategoryMapping(1618, TorznabCatType.Books, "         Historical books");
            AddCategoryMapping(1789, TorznabCatType.Books, "         Satire, humor (the book)");
            AddCategoryMapping(257, TorznabCatType.Books, "      Computer books");
            AddCategoryMapping(678, TorznabCatType.Books, "         Programming (Literature)");
            AddCategoryMapping(714, TorznabCatType.Books, "         Programs from Microsoft (literature)");
            AddCategoryMapping(715, TorznabCatType.Books, "         Other programs (literature)");
            AddCategoryMapping(716, TorznabCatType.Books, "         Mac OS; Linux, FreeBSD and other * NIX (literature)");
            AddCategoryMapping(717, TorznabCatType.Books, "         DBMS (literature)");
            AddCategoryMapping(718, TorznabCatType.Books, "         Web Design and Programming (Literature)");
            AddCategoryMapping(719, TorznabCatType.Books, "         Graphics, video processing (literature)");
            AddCategoryMapping(720, TorznabCatType.Books, "         Network / VoIP (literature)");
            AddCategoryMapping(721, TorznabCatType.Books, "         Hacking and Security (literature)");
            AddCategoryMapping(722, TorznabCatType.Books, "         Iron (book on a PC)");
            AddCategoryMapping(1215, TorznabCatType.Books, "         Engineering and science programs (literature)");
            AddCategoryMapping(1216, TorznabCatType.Books, "         Computer magazines and annexes");
            AddCategoryMapping(1791, TorznabCatType.Books, "            gaming magazines");
            AddCategoryMapping(1792, TorznabCatType.Books, "            Computer magazines");
            AddCategoryMapping(1217, TorznabCatType.Books, "         Disc applications to gaming magazines");
            AddCategoryMapping(311, TorznabCatType.Books, "      Comics");
            AddCategoryMapping(1218, TorznabCatType.Books, "         Comics in Russian");
            AddCategoryMapping(1219, TorznabCatType.Books, "         Marvel Comics publishing");
            AddCategoryMapping(1220, TorznabCatType.Books, "         DC Comics publishing");
            AddCategoryMapping(1221, TorznabCatType.Books, "         Comics from other publishers");
            AddCategoryMapping(1222, TorznabCatType.Books, "         Comics in other languages");
            AddCategoryMapping(1223, TorznabCatType.Books, "         Substandard distribution (Comics)");
            AddCategoryMapping(259, TorznabCatType.Books, "      Collections of books and libraries");
            AddCategoryMapping(983, TorznabCatType.Books, "         Libraries (mirror network libraries / collections)");
            AddCategoryMapping(984, TorznabCatType.Books, "         Thematic collections (collections)");
            AddCategoryMapping(1224, TorznabCatType.Books, "         Multidisciplinary collection (compilation)");
            AddCategoryMapping(1788, TorznabCatType.Books, "         Mnogoavtorskie collections, book series");
            AddCategoryMapping(1787, TorznabCatType.Books, "      Encyclopedias and dictionaries");
            AddCategoryMapping(1225, TorznabCatType.Books, "   Multimedia and online publications");
            AddCategoryMapping(1226, TorznabCatType.Books, "      Multimedia encyclopedia");
            AddCategoryMapping(1227, TorznabCatType.Books, "      Interactive tutorials and educational materials");
            AddCategoryMapping(1228, TorznabCatType.Books, "      Educational publications for children");
            AddCategoryMapping(1229, TorznabCatType.Books, "      Cooking. Floriculture. housekeeping");
            AddCategoryMapping(1230, TorznabCatType.Books, "      Culture. Art. History");
            AddCategoryMapping(497, TorznabCatType.Books, "training materials");
            AddCategoryMapping(1246, TorznabCatType.Books, "   Learning foreign languages");
            AddCategoryMapping(1248, TorznabCatType.Books, "      Foreign languages ??for children");
            AddCategoryMapping(1249, TorznabCatType.Books, "         English (for children)");
            AddCategoryMapping(1250, TorznabCatType.Books, "         Other European languages ??(for children)");
            AddCategoryMapping(1251, TorznabCatType.Books, "         Oriental languages ??(for children)");
            AddCategoryMapping(1252, TorznabCatType.Books, "         School books, the exam (for children)");
            AddCategoryMapping(1253, TorznabCatType.Books, "      Fiction in foreign languages");
            AddCategoryMapping(1254, TorznabCatType.Books, "         Fiction in English");
            AddCategoryMapping(1255, TorznabCatType.Books, "         Fiction French");
            AddCategoryMapping(1256, TorznabCatType.Books, "         Fiction in other European languages");
            AddCategoryMapping(1257, TorznabCatType.Books, "         Fiction in oriental languages");
            AddCategoryMapping(1258, TorznabCatType.Books, "      Foreign Language for Adults");
            AddCategoryMapping(1259, TorznabCatType.Books, "         English (for adults)");
            AddCategoryMapping(1260, TorznabCatType.Books, "         German");
            AddCategoryMapping(1261, TorznabCatType.Books, "         French");
            AddCategoryMapping(1262, TorznabCatType.Books, "         Spanish");
            AddCategoryMapping(1263, TorznabCatType.Books, "         Italian language");
            AddCategoryMapping(1264, TorznabCatType.Books, "         Other European languages");
            AddCategoryMapping(1265, TorznabCatType.Books, "         Arabic language");
            AddCategoryMapping(1266, TorznabCatType.Books, "         Chinese");
            AddCategoryMapping(1267, TorznabCatType.Books, "         Japanese");
            AddCategoryMapping(1268, TorznabCatType.Books, "         Other oriental languages");
            AddCategoryMapping(1269, TorznabCatType.Books, "         Russian as a foreign language");
            AddCategoryMapping(1270, TorznabCatType.Books, "         Multilanguage collections");
            AddCategoryMapping(1271, TorznabCatType.Books, "         Miscellaneous (foreign languages)");
            AddCategoryMapping(1867, TorznabCatType.Books, "         LIM-courses");
            AddCategoryMapping(1272, TorznabCatType.Books, "      Audio Books in foreign languages");
            AddCategoryMapping(1273, TorznabCatType.Books, "         Audiobooks in English");
            AddCategoryMapping(1274, TorznabCatType.Books, "         Audiobooks in German");
            AddCategoryMapping(1275, TorznabCatType.Books, "         Audiobooks in other languages");
            AddCategoryMapping(1623, TorznabCatType.Books, "      Educational audio materials");
            AddCategoryMapping(1247, TorznabCatType.Books, "   video tutorials");
            AddCategoryMapping(933, TorznabCatType.Books, "      Video tutorials and interactive training DVD");
            AddCategoryMapping(936, TorznabCatType.Books, "         Cooking (video tutorial)");
            AddCategoryMapping(1276, TorznabCatType.Books, "         Sport");
            AddCategoryMapping(934, TorznabCatType.Books, "         Fitness - Cardio, Strength Training");
            AddCategoryMapping(1277, TorznabCatType.Books, "         Fitness - Mind and Body");
            AddCategoryMapping(1278, TorznabCatType.Books, "         Extreme sports (video tutorial)");
            AddCategoryMapping(935, TorznabCatType.Books, "         Playing guitar");
            AddCategoryMapping(1279, TorznabCatType.Books, "         Body-building");
            AddCategoryMapping(1280, TorznabCatType.Books, "         Health practice");
            AddCategoryMapping(1281, TorznabCatType.Books, "         Yoga");
            AddCategoryMapping(1282, TorznabCatType.Books, "         Video and Snapshots");
            AddCategoryMapping(1283, TorznabCatType.Books, "         Personal care");
            AddCategoryMapping(1284, TorznabCatType.Books, "         Painting");
            AddCategoryMapping(1286, TorznabCatType.Books, "         Percussion instruments");
            AddCategoryMapping(1287, TorznabCatType.Books, "         Other musical instruments");
            AddCategoryMapping(1288, TorznabCatType.Books, "         Playing bass guitar");
            AddCategoryMapping(1289, TorznabCatType.Books, "         Ballroom dancing");
            AddCategoryMapping(1290, TorznabCatType.Books, "         Belly dance");
            AddCategoryMapping(1291, TorznabCatType.Books, "         Street and club dances");
            AddCategoryMapping(1292, TorznabCatType.Books, "         Dancing, miscellaneous");
            AddCategoryMapping(1295, TorznabCatType.Books, "         Tricks and stunts");
            AddCategoryMapping(1296, TorznabCatType.Books, "         Education");
            AddCategoryMapping(1297, TorznabCatType.Books, "         Business, Economics and Finance");
            AddCategoryMapping(1299, TorznabCatType.Books, "         Pregnancy, childbirth, motherhood");
            AddCategoryMapping(1300, TorznabCatType.Books, "         Educational video for children");
            AddCategoryMapping(1301, TorznabCatType.Books, "         Psychology (video)");
            AddCategoryMapping(1302, TorznabCatType.Books, "         Spirituality, self-development");
            AddCategoryMapping(1303, TorznabCatType.Books, "         Pickup, love");
            AddCategoryMapping(1304, TorznabCatType.Books, "         Construction, renovation and design");
            AddCategoryMapping(1305, TorznabCatType.Books, "         Wood and metal");
            AddCategoryMapping(1306, TorznabCatType.Books, "         Plants and Animals");
            AddCategoryMapping(1676, TorznabCatType.Books, "         Fishing and hunting");
            AddCategoryMapping(1293, TorznabCatType.Books, "            Hunting");
            AddCategoryMapping(1294, TorznabCatType.Books, "            Fishing and spearfishing");
            AddCategoryMapping(1307, TorznabCatType.Books, "         Miscellaneous (Video tutorials and educational interactive DVD)");
            AddCategoryMapping(1309, TorznabCatType.Books, "      Martial Arts (Video Tutorials)");
            AddCategoryMapping(1310, TorznabCatType.Books, "         Aikido and Aiki-jutsu");
            AddCategoryMapping(1311, TorznabCatType.Books, "         Wing Chun");
            AddCategoryMapping(1312, TorznabCatType.Books, "         Jujutsu");
            AddCategoryMapping(1313, TorznabCatType.Books, "         Judo and Sambo");
            AddCategoryMapping(1314, TorznabCatType.Books, "         Karate");
            AddCategoryMapping(1315, TorznabCatType.Books, "         knife fight");
            AddCategoryMapping(1316, TorznabCatType.Books, "         Work with weapon");
            AddCategoryMapping(1317, TorznabCatType.Books, "         Russian style");
            AddCategoryMapping(1318, TorznabCatType.Books, "         dogfight");
            AddCategoryMapping(1319, TorznabCatType.Books, "         composite style");
            AddCategoryMapping(1320, TorznabCatType.Books, "         shock styles");
            AddCategoryMapping(1321, TorznabCatType.Books, "         Wushu");
            AddCategoryMapping(1322, TorznabCatType.Books, "         Miscellaneous (Video Tutorials)");
            AddCategoryMapping(1323, TorznabCatType.Books, "      Computer video tutorials and interactive training DVD");
            AddCategoryMapping(1324, TorznabCatType.Books, "         Computer networks and security (video tutorial)");
            AddCategoryMapping(1325, TorznabCatType.Books, "         OS and Microsoft server software (video tutorial)");
            AddCategoryMapping(1326, TorznabCatType.Books, "         Microsoft Office software (video tutorial)");
            AddCategoryMapping(1327, TorznabCatType.Books, "         OS and UNIX-program (video tutorial)");
            AddCategoryMapping(1329, TorznabCatType.Books, "         Adobe Photoshop (video tutorial)");
            AddCategoryMapping(1330, TorznabCatType.Books, "         Autodesk Maya (video tutorial)");
            AddCategoryMapping(1331, TorznabCatType.Books, "         Autodesk 3ds Max (video tutorial)");
            AddCategoryMapping(1332, TorznabCatType.Books, "         Autodesk Softimage (XSI) (video tutorial)");
            AddCategoryMapping(1333, TorznabCatType.Books, "         ZBrush (video tutorial)");
            AddCategoryMapping(1334, TorznabCatType.Books, "         Flash, Flex and ActionScript (video tutorial)");
            AddCategoryMapping(1335, TorznabCatType.Books, "         2D-graphics (video tutorial)");
            AddCategoryMapping(1336, TorznabCatType.Books, "         3D-graphics (video tutorial)");
            AddCategoryMapping(1337, TorznabCatType.Books, "         Engineering and science programs (video tutorial)");
            AddCategoryMapping(1338, TorznabCatType.Books, "         Web-design (video tutorial)");
            AddCategoryMapping(1339, TorznabCatType.Books, "         Programming (video tutorial)");
            AddCategoryMapping(1340, TorznabCatType.Books, "         Software for Mac OS (video tutorial)");
            AddCategoryMapping(1341, TorznabCatType.Books, "         Working with video (video tutorial)");
            AddCategoryMapping(1342, TorznabCatType.Books, "         Working with sound (video tutorial)");
            AddCategoryMapping(1343, TorznabCatType.Books, "         Miscellaneous (Computer video tutorials)");
            AddCategoryMapping(1530, TorznabCatType.Other, "Subject forums");
            AddCategoryMapping(698, TorznabCatType.Other, "   Auto and Moto");
            AddCategoryMapping(699, TorznabCatType.Other, "      Repair and maintenance of vehicles");
            AddCategoryMapping(772, TorznabCatType.Other, "         The original selection of spare parts catalogs");
            AddCategoryMapping(1344, TorznabCatType.Other, "         Non-original spare parts catalogs for selection");
            AddCategoryMapping(1345, TorznabCatType.Other, "         diagnostic and repair programs");
            AddCategoryMapping(1346, TorznabCatType.Other, "         Tuning, chip tuning, tuning");
            AddCategoryMapping(700, TorznabCatType.Other, "         Books for repair / maintenance / operation of the vehicle");
            AddCategoryMapping(1349, TorznabCatType.Other, "         Multimediyki repair / maintenance / operation of the vehicle");
            AddCategoryMapping(1350, TorznabCatType.Other, "         Accounting, utilities, etc.");
            AddCategoryMapping(1351, TorznabCatType.Other, "         Virtual Driving School");
            AddCategoryMapping(1352, TorznabCatType.Other, "         Video lessons on driving vehicles");
            AddCategoryMapping(1353, TorznabCatType.Other, "         Video lessons on repair of vehicles");
            AddCategoryMapping(1354, TorznabCatType.Other, "         Magazines Auto / Moto");
            AddCategoryMapping(1355, TorznabCatType.Other, "         Water transport");
            AddCategoryMapping(1356, TorznabCatType.Other, "      Movies and television shows, car / moto");
            AddCategoryMapping(1357, TorznabCatType.Other, "         Documentary / educational films");
            AddCategoryMapping(1358, TorznabCatType.Other, "         entertainment shows");
            AddCategoryMapping(1359, TorznabCatType.Other, "         Top Gear / Top Gear");
            AddCategoryMapping(1360, TorznabCatType.Other, "         Test Drive / Reviews / Motor");
            AddCategoryMapping(1361, TorznabCatType.Other, "         Tuning / Fast and the Furious");
            AddCategoryMapping(600, TorznabCatType.Other, "   Medicine and Health");
            AddCategoryMapping(601, TorznabCatType.Other, "      Books, magazines and on medicine and health program");
            AddCategoryMapping(603, TorznabCatType.Other, "         Clinical medicine until 1980");
            AddCategoryMapping(604, TorznabCatType.Other, "         Clinical Medicine from 1980 to 2000");
            AddCategoryMapping(605, TorznabCatType.Other, "         Clinical Medicine since 2000");
            AddCategoryMapping(606, TorznabCatType.Other, "         The popular medical periodicals (newspapers and magazines)");
            AddCategoryMapping(607, TorznabCatType.Other, "         The scientific medical periodicals (newspapers and magazines)");
            AddCategoryMapping(608, TorznabCatType.Other, "         Life Sciences");
            AddCategoryMapping(609, TorznabCatType.Other, "         Pharmacy and Pharmacology");
            AddCategoryMapping(610, TorznabCatType.Other, "         Non-traditional, traditional medicine and popular books on health");
            AddCategoryMapping(611, TorznabCatType.Other, "         Veterinary, miscellaneous");
            AddCategoryMapping(612, TorznabCatType.Other, "         Thematic collections of books");
            AddCategoryMapping(613, TorznabCatType.Other, "         Audiobooks on medicine");
            AddCategoryMapping(614, TorznabCatType.Other, "         Medical software");
            AddCategoryMapping(602, TorznabCatType.Other, "      Tutorials, Doc. movies and TV shows on medicine");
            AddCategoryMapping(615, TorznabCatType.Other, "         Medicine and Dentistry");
            AddCategoryMapping(616, TorznabCatType.Other, "         Psychotherapy and clinical psychology");
            AddCategoryMapping(617, TorznabCatType.Other, "         Massage");
            AddCategoryMapping(618, TorznabCatType.Other, "         Health");
            AddCategoryMapping(619, TorznabCatType.Other, "         Documentary movies and TV shows on medicine");
            AddCategoryMapping(560, TorznabCatType.Other, "other");
            AddCategoryMapping(578, TorznabCatType.Other, "   Economy and Life");
            AddCategoryMapping(1558, TorznabCatType.Other, "   psychoactive audio programs");
            AddCategoryMapping(1560, TorznabCatType.Other, "   Avatars, Icons, Smileys, painting, drawing, sculpture, pictures, wallpaper, Photography, Digital Art");
            AddCategoryMapping(1651, TorznabCatType.Other, "      reproductions of paintings");
            AddCategoryMapping(1652, TorznabCatType.Other, "      Art photography");
            AddCategoryMapping(1653, TorznabCatType.Other, "      Contemporary photography");
            AddCategoryMapping(1654, TorznabCatType.Other, "      Collections of works of modern art painters");
            AddCategoryMapping(1655, TorznabCatType.Other, "      hand-drawn graphics");
            AddCategoryMapping(1656, TorznabCatType.Other, "      Computer graphics");
            AddCategoryMapping(1657, TorznabCatType.Other, "      Illustrations");
            AddCategoryMapping(1659, TorznabCatType.Other, "      Graphics (Other)");
            AddCategoryMapping(1562, TorznabCatType.Other, "      Amateur photos");
            AddCategoryMapping(1561, TorznabCatType.Other, "      Pictures");
            AddCategoryMapping(1563, TorznabCatType.Other, "      Photos of celebrities");
            AddCategoryMapping(996, TorznabCatType.Other, "      Desktop Wallpaper \\ Wallpapers");
            AddCategoryMapping(1578, TorznabCatType.Other, "      Wallpapers and themes for mobile devices");
            AddCategoryMapping(1559, TorznabCatType.Other, "      Avatars, Icons, Smileys");
            AddCategoryMapping(1564, TorznabCatType.Other, "   Audio");
            AddCategoryMapping(1801, TorznabCatType.Other, "      Mobile Audio");
            AddCategoryMapping(1565, TorznabCatType.Other, "   Video (Other)");
            AddCategoryMapping(1566, TorznabCatType.Other, "   Publications and educational materials (texts)");
            AddCategoryMapping(1567, TorznabCatType.Other, "   Sports (video)");
            AddCategoryMapping(1569, TorznabCatType.Other, "   Amateur videos");
        }

        public override async Task<IndexerConfigurationStatus> ApplyConfiguration(JToken configJson)
        {
            LoadValuesFromJson(configJson);

            var pairs = new Dictionary<string, string>
            {
                { "username", configData.Username.Value },
                { "password", configData.Password.Value },
                { "autologin", "on" },
                { "viewonline", "on" },
                { "login", "Вход" }
            };

            var result = await RequestLoginAndFollowRedirect(LoginUrl, pairs, null, true, null, LoginUrl, true);
            await ConfigureIfOK(result.Cookies, result.Content != null && result.Content.Contains("ucp.php?mode=logout&"), () =>
            {
                var errorMessage = result.Content;
                throw new ExceptionWithConfigData(errorMessage, configData);
            });
            return IndexerConfigurationStatus.RequiresTesting;
        }

        protected override async Task<IEnumerable<ReleaseInfo>> PerformQuery(TorznabQuery query)
        {
            var releases = new List<ReleaseInfo>();
            var searchString = query.GetQueryString();

            WebClientStringResult results = null;
            var queryCollection = new NameValueCollection();

            queryCollection.Add("st", "0");
            queryCollection.Add("sd", "d");
            queryCollection.Add("sk", "t");
            queryCollection.Add("tracker_search", "torrent");
            queryCollection.Add("t", "0");
            queryCollection.Add("submit", "Search");
            queryCollection.Add("sr", "topics");
            //queryCollection.Add("sr", "posts");
            //queryCollection.Add("ch", "99999");

            // if the search string is empty use the getnew view
            if (string.IsNullOrWhiteSpace(searchString))
            {
                queryCollection.Add("search_id", "newposts");
            }
            else // use the normal search
            {
                searchString = searchString.Replace("-", " ");
                queryCollection.Add("terms", "all");
                queryCollection.Add("keywords", searchString);
                queryCollection.Add("author", "");
                queryCollection.Add("sc", "1");
                queryCollection.Add("sf", "titleonly");
            }

            var searchUrl = SearchUrl + "?" + queryCollection.GetQueryString();
            results = await RequestStringWithCookies(searchUrl);
            if (!results.Content.Contains("ucp.php?mode=logout"))
            {
                await ApplyConfiguration(null);
                results = await RequestStringWithCookies(searchUrl);
            }
            try
            {
                string RowsSelector = "ul.topics > li";

                var SearchResultParser = new HtmlParser();
                var SearchResultDocument = SearchResultParser.Parse(results.Content);
                var Rows = SearchResultDocument.QuerySelectorAll(RowsSelector);
                foreach (var Row in Rows)
                {
                    try
                    {
                        var release = new ReleaseInfo();

                        release.MinimumRatio = 1;
                        release.MinimumSeedTime = 0;

                        var qDetailsLink = Row.QuerySelector("a.topictitle");
                        var qDownloadLink = Row.QuerySelector("a[href^=\"./download/file.php?id=\"]");

                        release.Title = qDetailsLink.TextContent;
                        release.Comments = new Uri(SiteLink + qDetailsLink.GetAttribute("href"));
                        release.Link = new Uri(SiteLink + qDownloadLink.GetAttribute("href"));
                        release.Guid = release.Comments;

                        release.Seeders = ParseUtil.CoerceInt(Row.QuerySelector("span.seed").TextContent);
                        release.Peers = ParseUtil.CoerceInt(Row.QuerySelector("span.leech").TextContent) + release.Seeders;
                        release.Grabs = ParseUtil.CoerceLong(Row.QuerySelector("span.complet").TextContent);

                        var author = Row.QuerySelector("a[href^=\"./memberlist.php?mode=viewprofile&\"]");
                        var timestr = author.NextSibling.NodeValue.Substring(3).Split('\n')[0].Trim();

                        timestr = timestr.Replace("менее минуты назад", "now");
                        timestr = timestr.Replace("назад", "ago");
                        timestr = timestr.Replace("минуту", "minute");
                        timestr = timestr.Replace("минуты", "minutes");
                        timestr = timestr.Replace("минут", "minutes");

                        timestr = timestr.Replace("Сегодня", "Today");
                        timestr = timestr.Replace("Вчера", "Yesterday"); // untested

                        timestr = timestr.Replace("янв", "Jan");
                        timestr = timestr.Replace("фев", "Feb");
                        timestr = timestr.Replace("мар", "Mar");
                        timestr = timestr.Replace("апр", "Apr");
                        timestr = timestr.Replace("май", "May");
                        timestr = timestr.Replace("июн", "Jun");
                        timestr = timestr.Replace("июл", "Jul");
                        timestr = timestr.Replace("авг", "Aug");
                        timestr = timestr.Replace("сен", "Sep");
                        timestr = timestr.Replace("окт", "Oct");
                        timestr = timestr.Replace("ноя", "Nov");
                        timestr = timestr.Replace("дек", "Dec");
                        release.PublishDate = DateTimeUtil.FromUnknown(timestr, "UK");

                        var forum = Row.QuerySelector("a[href^=\"./viewforum.php?f=\"]");
                        var forumid = forum.GetAttribute("href").Split('=')[1];
                        release.Category = MapTrackerCatToNewznab(forumid);

                        var size = forum.NextElementSibling;
                        var sizestr = size.TextContent;
                        sizestr = sizestr.Replace("ГБ", "GB");
                        sizestr = sizestr.Replace("МБ", "MB");
                        sizestr = sizestr.Replace("КБ", "KB"); // untested
                        release.Size = ReleaseInfo.GetBytes(sizestr);

                        release.DownloadVolumeFactor = 1;
                        release.UploadVolumeFactor = 1;

                        releases.Add(release);
                    }
                    catch (Exception ex)
                    {
                        logger.Error(string.Format("{0}: Error while parsing row '{1}':\n\n{2}", ID, Row.OuterHtml, ex));
                    }
                }
            }
            catch (Exception ex)
            {
                OnParseError(results.Content, ex);
            }

            return releases;
        }
    }
}
